@@ -27,4 +27,5 @@ void extractcgiargs(void);
 void extractpostargs(void);
 
 /* see if a host is allowed per the hosts.conf */
-int checkhost(char *check, char **desc);
+int checkhost(const char *host, char **desc);
+
