@@ -14,7 +14,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.              
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
 struct {
@@ -32,5 +32,6 @@ struct {
 	{ "TRIM",	"VOLTAGE TRIM",		1	},
 	{ "BOOST",	"VOLTAGE BOOST",	1	},
 	{ "CAL",	"CALIBRATION",		1	},
+	{ "BYPASS",	"BYPASS",		2	},
 	{ NULL,		NULL,			0	}
 };
