@@ -27,6 +27,7 @@
 #include <unistd.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
+#include <arpa/inet.h>
 
 #include "upsclient.h"
 
@@ -89,68 +90,73 @@ struct {
 };
 
 /* make sure we're using a struct that's been through upscli_connect */
-static int upscli_checkmagic(UPSCONN *ups)
+static int upscli_checkmagic(UPSCONN_t *ups)
 {
-	if (!ups)
+	if (!ups) {
 		return 0;
+	}
 
-	if (ups->upsclient_magic != UPSCLIENT_MAGIC)
+	if (ups->upsclient_magic != UPSCLIENT_MAGIC) {
 		return 0;
+	}
 
 	return 1;
 }
 
-const char *upscli_strerror(UPSCONN *ups)
+const char *upscli_strerror(UPSCONN_t *ups)
 {
 #ifdef HAVE_SSL
 	unsigned long	err;
 	char	sslbuf[UPSCLI_ERRBUF_LEN];
 #endif
 
-	if (!ups)
+	if (!ups) {
 		return upscli_errlist[UPSCLI_ERR_INVALIDARG].str;
+	}
 
-	if (!upscli_checkmagic(ups))
+	if (!upscli_checkmagic(ups)) {
 		return upscli_errlist[UPSCLI_ERR_INVALIDARG].str;
+	}
 
-	if (ups->upserror > UPSCLI_ERR_MAX)
+	if (ups->upserror > UPSCLI_ERR_MAX) {
 		return "Invalid error number";
+	}
 
 	switch (upscli_errlist[ups->upserror].flags) {
 
-		case 0:		/* simple error */
-			return upscli_errlist[ups->upserror].str;
+	case 0:		/* simple error */
+		return upscli_errlist[ups->upserror].str;
 
-		case 1:		/* add message from system's strerror */
-			snprintf(ups->errbuf, UPSCLI_ERRBUF_LEN,
-				upscli_errlist[ups->upserror].str,
-				strerror(ups->syserrno));
-			return ups->errbuf;
+	case 1:		/* add message from system's strerror */
+		snprintf(ups->errbuf, UPSCLI_ERRBUF_LEN,
+			upscli_errlist[ups->upserror].str,
+			strerror(ups->syserrno));
+		return ups->errbuf;
 
-		case 2:		/* SSL error */
+	case 2:		/* SSL error */
 #ifdef HAVE_SSL
-			err = ERR_get_error();
-			if (err) {
-				ERR_error_string(err, sslbuf);
-				snprintf(ups->errbuf, UPSCLI_ERRBUF_LEN,
-					upscli_errlist[ups->upserror].str,
-					sslbuf);
-			} else {
-				snprintf(ups->errbuf, UPSCLI_ERRBUF_LEN,
-					upscli_errlist[ups->upserror].str,
-					"peer disconnected");
-			}
-#else
-			snprintf(ups->errbuf, UPSCLI_ERRBUF_LEN, 
-				"SSL error, but SSL wasn't enabled at compile-time");
-#endif	/* HAVE_SSL */
-			return ups->errbuf;
-
-		case 3:		/* parsing (parseconf) error */
+		err = ERR_get_error();
+		if (err) {
+			ERR_error_string(err, sslbuf);
 			snprintf(ups->errbuf, UPSCLI_ERRBUF_LEN,
 				upscli_errlist[ups->upserror].str,
-				ups->pc_ctx->errmsg);
-			return ups->errbuf;
+				sslbuf);
+		} else {
+			snprintf(ups->errbuf, UPSCLI_ERRBUF_LEN,
+				upscli_errlist[ups->upserror].str,
+				"peer disconnected");
+		}
+#else
+		snprintf(ups->errbuf, UPSCLI_ERRBUF_LEN, 
+			"SSL error, but SSL wasn't enabled at compile-time");
+#endif	/* HAVE_SSL */
+		return ups->errbuf;
+
+	case 3:		/* parsing (parseconf) error */
+		snprintf(ups->errbuf, UPSCLI_ERRBUF_LEN,
+			upscli_errlist[ups->upserror].str,
+			ups->pc_ctx->errmsg);
+		return ups->errbuf;
 	}
 
 	/* fallthrough */
@@ -161,7 +167,7 @@ const char *upscli_strerror(UPSCONN *ups)
 	return ups->errbuf;
 }
 
-static void upscli_closefd(UPSCONN *ups)
+static void upscli_closefd(UPSCONN_t *ups)
 {
 #ifdef HAVE_SSL
 	if (ups->ssl) {
@@ -182,7 +188,7 @@ static void upscli_closefd(UPSCONN *ups)
 }
 
 /* internal: abstract the SSL calls for the other functions */
-static int net_read(UPSCONN *ups, char *buf, size_t buflen)
+static int net_read(UPSCONN_t *ups, char *buf, size_t buflen)
 {
 	int	ret;
 
@@ -219,7 +225,7 @@ static int net_read(UPSCONN *ups, char *buf, size_t buflen)
 	return ret;
 }
 
-static int net_write(UPSCONN *ups, const char *buf, size_t count)
+static int net_write(UPSCONN_t *ups, const char *buf, size_t count)
 {
 	int	ret;
 
@@ -249,14 +255,15 @@ static int net_write(UPSCONN *ups, const char *buf, size_t count)
 }
 
 /* internal: bring back a line (up to LF or buflen bytes) */
-static int upscli_read(UPSCONN *ups, char *buf, size_t buflen)
+static int upscli_read(UPSCONN_t *ups, char *buf, size_t buflen)
 {
 	int	ret;
 	size_t	numrec = 0;
 	char	ch;
 
-	if (!ups)
+	if (!ups) {
 		return -1;
+	}
 
 	if ((!buf) || (buflen < 1) || (ups->fd == -1)) {
 		ups->upserror = UPSCLI_ERR_INVALIDARG;
@@ -271,8 +278,9 @@ static int upscli_read(UPSCONN *ups, char *buf, size_t buflen)
 	for (;;) {
 		ret = net_read(ups, &ch, 1);
 
-		if (ret < 1)
+		if (ret < 1) {
 			return -1;	
+		}
 
 		if ((ch == 10) || (numrec == (buflen - 1))) {
 
@@ -289,15 +297,16 @@ static int upscli_read(UPSCONN *ups, char *buf, size_t buflen)
 	
 /* stub first */
 #ifndef HAVE_SSL
-static int upscli_sslinit(UPSCONN *ups)
+static int upscli_sslinit(UPSCONN_t *ups)
 {
 	return 0;		/* not supported */
 }
 
-int upscli_sslcert(UPSCONN *ups, const char *dir, const char *file, int verify)
+int upscli_sslcert(UPSCONN_t *ups, const char *dir, const char *file, int verify)
 {
-	if (!ups)
+	if (!ups) {
 		return -1;
+	}
 
 	/* if forcing the verification, this fails since we have no SSL */
 	if (verify == 1) {
@@ -310,13 +319,14 @@ int upscli_sslcert(UPSCONN *ups, const char *dir, const char *file, int verify)
 
 #else
 
-static int upscli_sslinit(UPSCONN *ups)
+static int upscli_sslinit(UPSCONN_t *ups)
 {
 	int	ret;
 	char	buf[UPSCLI_NETBUF_LEN];
 
-	if (!ups)
+	if (!ups) {
 		return -1;
+	}
 
 	if (!upscli_checkmagic(ups)) {
 		ups->upserror = UPSCLI_ERR_INVALIDARG;
@@ -355,8 +365,9 @@ static int upscli_sslinit(UPSCONN *ups)
 		return -1;
 	}
 
-	if (strncmp(buf, "OK STARTTLS", 11) != 0)
+	if (strncmp(buf, "OK STARTTLS", 11) != 0) {
 		return 0;		/* not supported */
+	}
 
 	/* upsd is happy, so let's crank up the client */
 
@@ -365,18 +376,21 @@ static int upscli_sslinit(UPSCONN *ups)
 
 	ups->ssl_ctx = SSL_CTX_new(TLSv1_client_method());
 
-	if (!ups->ssl_ctx)
+	if (!ups->ssl_ctx) {
 		return 0;
-	
+	}
+
 	ups->ssl = SSL_new(ups->ssl_ctx);
 
-	if (!ups->ssl)
+	if (!ups->ssl) {
 		return 0;
+	}
 
 	ret = SSL_set_fd(ups->ssl, ups->fd);
 
-	if (ret != 1)
+	if (ret != 1) {
 		return -1;
+	}
 
 	SSL_set_connect_state(ups->ssl);
 
@@ -384,12 +398,13 @@ static int upscli_sslinit(UPSCONN *ups)
 }
 
 /* set the paths for the certs to verify the server */
-int upscli_sslcert(UPSCONN *ups, const char *file, const char *path, int verify)
+int upscli_sslcert(UPSCONN_t *ups, const char *file, const char *path, int verify)
 {
 	int	ret, ssl_mode = SSL_VERIFY_NONE;
 
-	if (!ups)
+	if (!ups) {
 		return -1;
+	}
 
 	if (!ups->ssl_ctx) {
 		ups->upserror = UPSCLI_ERR_INVALIDARG;
@@ -397,12 +412,16 @@ int upscli_sslcert(UPSCONN *ups, const char *file, const char *path, int verify)
 	}
 
 	switch(verify) {
-		case 0: ssl_mode = SSL_VERIFY_NONE; break;
-		case 1: ssl_mode = SSL_VERIFY_PEER; break;
 
-		default:
-			ups->upserror = UPSCLI_ERR_INVALIDARG;
-			return -1;
+	case 0:
+		ssl_mode = SSL_VERIFY_NONE;
+		break;
+	case 1:
+		ssl_mode = SSL_VERIFY_PEER;
+		break;
+	default:
+		ups->upserror = UPSCLI_ERR_INVALIDARG;
+		return -1;
 	}
 
 	ret = SSL_CTX_load_verify_locations(ups->ssl_ctx, file, path);
@@ -418,11 +437,17 @@ int upscli_sslcert(UPSCONN *ups, const char *file, const char *path, int verify)
 }
 
 #endif	/* HAVE_SSL */
-	
-int upscli_connect(UPSCONN *ups, const char *host, int port, int flags)
+
+int upscli_connect(UPSCONN_t *ups, const char *host, int port, int flags)
 {
-	struct	sockaddr_in	local, server;
-	struct	hostent	*serv;
+#ifndef	HAVE_IPV6
+	struct sockaddr_in	local, server;
+	struct hostent		*serv;
+#else
+	struct addrinfo	hints, *res, *ai;
+	char			sport[NI_MAXSERV];
+	int			v;
+#endif
 
 	/* clear out any lingering junk */
 	ups->fd = -1;
@@ -432,11 +457,9 @@ int upscli_connect(UPSCONN *ups, const char *host, int port, int flags)
 	ups->syserrno = 0;
 	ups->upsclient_magic = UPSCLIENT_MAGIC;
 
-	ups->pc_ctx = malloc(sizeof(PCONF_CTX));
-
-	if (!ups->pc_ctx) {
+	if ((ups->pc_ctx = malloc(sizeof(PCONF_CTX_t))) == NULL) {
 		ups->upserror = UPSCLI_ERR_NOMEM;
-		return -1;		
+		return -1;
 	}
 
 	pconf_init(ups->pc_ctx, NULL);
@@ -449,10 +472,23 @@ int upscli_connect(UPSCONN *ups, const char *host, int port, int flags)
 		return -1;
 	}
 
-	if ((serv = gethostbyname(host)) == (struct hostent *) NULL) {
+#ifndef	HAVE_IPV6
+	serv = gethostbyname(host);
 
-		ups->upserror = UPSCLI_ERR_NOSUCHHOST;
-		return -1;
+	if (serv == NULL) {
+		struct  in_addr	listenaddr;
+
+		if (!inet_aton(host, &listenaddr)) {
+			ups->upserror = UPSCLI_ERR_NOSUCHHOST;
+			return -1;
+		}
+
+		serv = gethostbyaddr(&listenaddr, sizeof(listenaddr), AF_INET);
+
+		if (serv == NULL) {
+			ups->upserror = UPSCLI_ERR_NOSUCHHOST;
+			return -1;
+		}
 	}
 
 	if ((ups->fd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
@@ -471,34 +507,107 @@ int upscli_connect(UPSCONN *ups, const char *host, int port, int flags)
 
 	memcpy(&server.sin_addr, serv->h_addr, serv->h_length);
 
-	if (bind(ups->fd, (struct sockaddr *) &local, 
-		sizeof(struct sockaddr_in)) == -1) {
+	if (bind(ups->fd, (struct sockaddr *) &local, sizeof(struct sockaddr_in)) == -1) {
 		ups->upserror = UPSCLI_ERR_BINDFAILURE;
 		ups->syserrno = errno;
 		close(ups->fd);
 		ups->fd = -1;
-
 		return -1;
 	}
 
-	if (connect(ups->fd, (struct sockaddr *) &server, 
-		sizeof(struct sockaddr_in)) == -1) {
+	if (connect(ups->fd, (struct sockaddr *) &server, sizeof(struct sockaddr_in)) == -1) {
 		ups->upserror = UPSCLI_ERR_CONNFAILURE;
 		ups->syserrno = errno;
 		close(ups->fd);
 		ups->fd = -1;
+		return -1;
+	}
+#else
+	snprintf(sport, NI_MAXSERV, "%hu", (unsigned short int)port);
 
+	memset (&hints, 0, sizeof (struct addrinfo));
+
+	if (flags & UPSCLI_CONN_INET6) {
+		hints.ai_family = AF_INET6;
+	} else if (flags & UPSCLI_CONN_INET) {
+		hints.ai_family = AF_INET;
+	} else {
+		hints.ai_family = AF_UNSPEC;
+	}
+
+	hints.ai_socktype = SOCK_STREAM;
+	hints.ai_protocol = IPPROTO_TCP;
+
+	while ((v = getaddrinfo(host, sport, &hints, &res)) != 0) {
+		switch (v) {
+		case EAI_AGAIN:
+			continue;
+		case EAI_NONAME:
+			ups->upserror = UPSCLI_ERR_NOSUCHHOST;
+			return -1;
+		case EAI_MEMORY:
+			ups->upserror = UPSCLI_ERR_NOMEM;
+			return -1;
+		case EAI_SYSTEM:
+			ups->syserrno = errno;
+			break;
+		}
+
+		ups->upserror = UPSCLI_ERR_UNKNOWN;
 		return -1;
 	}
 
-	/* don't use xstrdup for cleaner linking (fewer dependencies) */
-	ups->host = strdup(host);
+	for (ai = res; ai != NULL; ai = ai->ai_next) {
+		int	sock_fd;
 
-	if (!ups->host) {
+		if ((sock_fd = socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol)) < 0) {
+			switch (errno) {
+			case EAFNOSUPPORT:
+			case EINVAL:
+                                break;
+			default:
+				ups->upserror = UPSCLI_ERR_SOCKFAILURE;
+				ups->syserrno = errno;
+			}
+			continue;
+		}
+
+		while ((v = connect(sock_fd, ai->ai_addr, ai->ai_addrlen)) < 0) {
+			switch (errno) {
+			case EAFNOSUPPORT:
+				break;
+			case EINTR:
+			case EAGAIN:
+				continue;
+			default:
+				ups->upserror = UPSCLI_ERR_CONNFAILURE;
+				ups->syserrno = errno;
+			}
+			break;
+		}
+
+		if (v < 0) {
+			close(sock_fd);
+			continue;
+		}
+
+		ups->fd = sock_fd;
+		ups->upserror = 0;
+		ups->syserrno = 0;
+		break;
+	}		
+
+	freeaddrinfo(res);
+
+	if (ups->fd < 0) {
+		return -1;
+	}
+#endif
+
+	if ((ups->host = strdup(host)) == NULL) {
+		ups->upserror = UPSCLI_ERR_NOMEM;
 		close(ups->fd);
 		ups->fd = -1;
-
-		ups->upserror = UPSCLI_ERR_NOMEM;
 		return -1;
 	}
 
@@ -508,21 +617,19 @@ int upscli_connect(UPSCONN *ups, const char *host, int port, int flags)
 		upscli_sslinit(ups);
 
 		/* see if something made us die inside sslinit */
-		if (ups->upserror != 0)
-			return -1;
-	}
-
-	if (flags & UPSCLI_CONN_REQSSL) {
-		if (upscli_sslinit(ups) != 1) {
-			ups->upserror = UPSCLI_ERR_SSLFAIL;
-			upscli_closefd(ups);
+		if (ups->upserror != 0) {
 			return -1;
 		}
 	}
 
+	if ((flags & UPSCLI_CONN_REQSSL) && (upscli_sslinit(ups) != 1)) {
+		ups->upserror = UPSCLI_ERR_SSLFAIL;
+		upscli_closefd(ups);
+		return -1;
+	}
+		
 	return 0;
 }
-
 /* map upsd error strings back to upsclient internal numbers */
 static struct {
 	int	errnum;
@@ -558,12 +665,13 @@ static struct {
 	{ 0,			NULL,		}
 };
 
-static int upscli_errcheck(UPSCONN *ups, char *buf)
+static int upscli_errcheck(UPSCONN_t *ups, char *buf)
 {
 	int	i;
 
-	if (!ups)
+	if (!ups) {
 		return -1;
+	}
 
 	if (!buf) {
 		ups->upserror = UPSCLI_ERR_INVALIDARG;
@@ -571,8 +679,9 @@ static int upscli_errcheck(UPSCONN *ups, char *buf)
 	}
 
 	/* see if it's even an error now */
-	if (strncmp(buf, "ERR", 3) != 0)
+	if (strncmp(buf, "ERR", 3) != 0) {
 		return 0;
+	}
 
 	/* look it up in the table */
 	for (i = 0; upsd_errlist[i].text != NULL; i++) {
@@ -602,10 +711,11 @@ static void build_cmd(char *buf, size_t bufsize, const char *cmdname,
 	/* encode all arguments so they arrive intact */
 	for (i = 0; i < numarg; i++) {
 
-		if (strchr(arg[i], ' '))
+		if (strchr(arg[i], ' ')) {
 			format = " \"%s\"";	/* wrap in "" */
-		else
+		} else {
 			format = " %s";
+		}
 
 		/* snprintfcat would tie us to common */
 
@@ -634,7 +744,7 @@ static int verify_resp(int num, const char **q, char **a)
 	return 1;	/* OK */
 }
 
-int upscli_get(UPSCONN *ups, unsigned int numq, const char **query, 
+int upscli_get(UPSCONN_t *ups, unsigned int numq, const char **query, 
 		unsigned int *numa, char ***answer)
 {
 	int	ret;
@@ -650,14 +760,17 @@ int upscli_get(UPSCONN *ups, unsigned int numq, const char **query,
 
 	ret = net_write(ups, cmd, strlen(cmd));
 
-	if (ret < 0)
+	if (ret < 0) {
 		return -1;
+	}
 
-	if (upscli_read(ups, tmp, sizeof(tmp)) != 0)
+	if (upscli_read(ups, tmp, sizeof(tmp)) != 0) {
 		return -1;
+	}
 
-	if (upscli_errcheck(ups, tmp) != 0)
+	if (upscli_errcheck(ups, tmp) != 0) {
 		return -1;
+	}
 
 	if (!pconf_line(ups->pc_ctx, tmp)) {
 		ups->upserror = UPSCLI_ERR_PARSE;
@@ -683,7 +796,7 @@ int upscli_get(UPSCONN *ups, unsigned int numq, const char **query,
 	return 0;
 }
 
-int upscli_list_start(UPSCONN *ups, unsigned int numq, const char **query)
+int upscli_list_start(UPSCONN_t *ups, unsigned int numq, const char **query)
 {
 	int	ret;
 	char	cmd[UPSCLI_NETBUF_LEN], tmp[UPSCLI_NETBUF_LEN];
@@ -698,14 +811,17 @@ int upscli_list_start(UPSCONN *ups, unsigned int numq, const char **query)
 
 	ret = net_write(ups, cmd, strlen(cmd));
 
-	if (ret < 0)
+	if (ret < 0) {
 		return -1;
+	}
 
-	if (upscli_read(ups, tmp, sizeof(tmp)) != 0)
+	if (upscli_read(ups, tmp, sizeof(tmp)) != 0) {
 		return -1;
+	}
 
-	if (upscli_errcheck(ups, tmp) != 0)
+	if (upscli_errcheck(ups, tmp) != 0) {
 		return -1;
+	}
 
 	if (!pconf_line(ups->pc_ctx, tmp)) {
 		ups->upserror = UPSCLI_ERR_PARSE;
@@ -737,16 +853,18 @@ int upscli_list_start(UPSCONN *ups, unsigned int numq, const char **query)
 	return 0;
 }
 
-int upscli_list_next(UPSCONN *ups, unsigned int numq, const char **query, 
+int upscli_list_next(UPSCONN_t *ups, unsigned int numq, const char **query, 
 		unsigned int *numa, char ***answer)
 {
 	char	tmp[UPSCLI_NETBUF_LEN];
 
-	if (upscli_read(ups, tmp, sizeof(tmp)) != 0)
+	if (upscli_read(ups, tmp, sizeof(tmp)) != 0) {
 		return -1;
+	}
 
-	if (upscli_errcheck(ups, tmp) != 0)
+	if (upscli_errcheck(ups, tmp) != 0) {
 		return -1;
+	}
 
 	if (!pconf_line(ups->pc_ctx, tmp)) {
 		ups->upserror = UPSCLI_ERR_PARSE;
@@ -780,12 +898,13 @@ int upscli_list_next(UPSCONN *ups, unsigned int numq, const char **query,
 	return 1;
 }
 
-int upscli_sendline(UPSCONN *ups, const char *buf, size_t buflen)
+int upscli_sendline(UPSCONN_t *ups, const char *buf, size_t buflen)
 {
 	int	ret;
 
-	if (!ups)
+	if (!ups) {
 		return -1;
+	}
 
 	if ((!buf) || (buflen < 1) || (ups->fd == -1)) {
 		ups->upserror = UPSCLI_ERR_INVALIDARG;
@@ -799,18 +918,20 @@ int upscli_sendline(UPSCONN *ups, const char *buf, size_t buflen)
 
 	ret = net_write(ups, buf, buflen);
 
-	if (ret < 0)
+	if (ret < 0) {
 		return -1;
+	}
 
 	return 0;
 }
 
-int upscli_readline(UPSCONN *ups, char *buf, size_t buflen)
+int upscli_readline(UPSCONN_t *ups, char *buf, size_t buflen)
 {
 	char	tmp[LARGEBUF];
 
-	if (!ups)
+	if (!ups) {
 		return -1;
+	}
 
 	if ((!buf) || (buflen < 1) || (ups->fd == -1)) {
 		ups->upserror = UPSCLI_ERR_INVALIDARG;
@@ -822,82 +943,118 @@ int upscli_readline(UPSCONN *ups, char *buf, size_t buflen)
 		return -1;
 	}
 
-	if (upscli_read(ups, tmp, sizeof(tmp)) != 0)
+	if (upscli_read(ups, tmp, sizeof(tmp)) != 0) {
 		return -1;
+	}
 
-	if (upscli_errcheck(ups, tmp) != 0)
+	if (upscli_errcheck(ups, tmp) != 0) {
 		return -1;
+	}
 
 	snprintf(buf, buflen, "%s", tmp);
 
 	return 0;
 }
 
-/* split [upsname@]hostname[:port] into separate components */
-int upscli_splitname(const char *buf, char **upsname, char **hostname,
-			int *port)
+/* split upsname[@hostname[:port]] into separate components */
+int upscli_splitname(const char *buf, char **upsname, char **hostname, int *port)
 {
-	char	tmp[SMALLBUF], *ptr, *ap, *cp;
+	char	*s, tmp[SMALLBUF];
 
-	if ((!buf) || (!upsname) || (!hostname) || (!port))
-		return -1;
-
-	snprintf(tmp, sizeof(tmp), "%s", buf);
-
-	ap = strchr(tmp, '@');
-
-	if (!ap) {
-		fprintf(stderr, "upscli_splitname: no UPS name specified (upsname@hostname)\n");
+	/* paranoia */
+	if ((!buf) || (!upsname) || (!hostname) || (!port)) {
 		return -1;
 	}
 
-	*ap++ = '\0';
-	*upsname = strdup(tmp);
+	if (snprintf(tmp, SMALLBUF, "%s", buf) < 1) {
+		fprintf(stderr, "upscli_splitname: can't parse empty string\n");
+		return -1;
+	}
 
-	if (!*upsname) {
+	s = strchr(tmp, '@');
+
+	if ((*upsname = strdup(strtok(tmp, "@"))) == NULL) {
 		fprintf(stderr, "upscli_splitname: strdup failed\n");
 		return -1;
 	}
 
-	ptr = ap;
-
-	cp = strchr(ptr, ':');
-
-	if (cp) {
-		*cp++ = '\0';
-		*hostname = strdup(ptr);
-
-		if (!*hostname) {
-			fprintf(stderr, "upscli_splitname: strdup failed\n");
-			return -1;
-		}
-
-		ptr = cp;
-
-		*port = strtol(ptr, (char **) NULL, 10);
-
-	} else {
-
-		*hostname = strdup(ptr);
-
-		if (!*hostname) {
+	/* only a upsname is specified, fill in defaults */
+	if (s == NULL) {
+		if ((*hostname = strdup("localhost")) == NULL) {
 			fprintf(stderr, "upscli_splitname: strdup failed\n");
 			return -1;
 		}
 
 		*port = PORT;
+		return 0;
+	}
+
+	return upscli_splitaddr(s+1, hostname, port);
+}
+
+/* split hostname[:port] into separate components */
+int upscli_splitaddr(const char *buf, char **hostname, int *port)
+{
+	char	*s, tmp[SMALLBUF];
+
+	/* paranoia */
+	if ((!buf) || (!hostname) || (!port)) {
+		return -1;
+	}
+
+	if (snprintf(tmp, SMALLBUF, "%s", buf) < 1) {
+		fprintf(stderr, "upscli_splitaddr: can't parse empty string\n");
+		return -1;
+	}
+
+	if (*tmp == '[') {
+		if (strchr(tmp, ']') == NULL) {
+			fprintf(stderr, "upscli_splitaddr: missing closing bracket in [domain literal]\n");
+			return -1;
+		}
+
+		if ((*hostname = strdup(strtok(tmp+1, "]"))) == NULL) {
+			fprintf(stderr, "upscli_splitaddr: strdup failed\n");
+			return -1;
+		}
+
+		/* no port specified, use default */
+		if (((s = strtok(NULL, "\0")) == NULL) || (*s != ':')) {
+			*port = PORT;
+			return 0;
+		}
+	} else {
+		s = strchr(tmp, ':');
+
+		if ((*hostname = strdup(strtok(tmp, ":"))) == NULL) {
+			fprintf(stderr, "upscli_splitaddr: strdup failed\n");
+			return -1;
+		}
+
+		/* no port specified, use default */
+		if (s == NULL) {
+			*port = PORT;
+			return 0;
+		}
+	}
+
+	if ((*(++s) == '\0') || ((*port = strtol(s, NULL, 10)) < 1 )) {
+		fprintf(stderr, "upscli_splitaddr: no port specified after ':' separator\n");
+		return -1;
 	}
 
 	return 0;
 }
 
-int upscli_disconnect(UPSCONN *ups)
+int upscli_disconnect(UPSCONN_t *ups)
 {
-	if (!ups)
+	if (!ups) {
 		return -1;
+	}
 
-	if (!upscli_checkmagic(ups))
+	if (!upscli_checkmagic(ups)) {
 		return -1;
+	}
 
 	if (ups->fd != -1) {
 
@@ -913,8 +1070,7 @@ int upscli_disconnect(UPSCONN *ups)
 		ups->pc_ctx = NULL;
 	}
 
-	if (ups->host)
-		free(ups->host);
+	free(ups->host);
 
 	ups->host = NULL;
 	ups->upsclient_magic = 0;
@@ -922,38 +1078,45 @@ int upscli_disconnect(UPSCONN *ups)
 	return 0;
 }
 
-int upscli_fd(UPSCONN *ups)
+int upscli_fd(UPSCONN_t *ups)
 {
-	if (!ups)
+	if (!ups) {
 		return -1;
+	}
 
-	if (!upscli_checkmagic(ups))
+	if (!upscli_checkmagic(ups)) {
 		return -1;
+	}
 
 	return ups->fd;
 }
 
-int upscli_upserror(UPSCONN *ups)
+int upscli_upserror(UPSCONN_t *ups)
 {
-	if (!ups)
+	if (!ups) {
 		return -1;
+	}
 
-	if (!upscli_checkmagic(ups))
+	if (!upscli_checkmagic(ups)) {
 		return -1;
+	}
 
 	return ups->upserror;
 }
 
-int upscli_ssl(UPSCONN *ups)
+int upscli_ssl(UPSCONN_t *ups)
 {
-	if (!ups)
+	if (!ups) {
 		return -1;
+	}
 
-	if (!upscli_checkmagic(ups))
+	if (!upscli_checkmagic(ups)) {
 		return -1;
+	}
 
-	if (ups->ssl)
+	if (ups->ssl) {
 		return 1;
+	}
 
 	return 0;
 }
