@@ -43,7 +43,7 @@ typedef struct {
 	int	syserrno;
 	int	upsclient_magic;
 
-	PCONF_CTX	*pc_ctx;
+	PCONF_CTX_t	*pc_ctx;
 
 	char	errbuf[UPSCLI_ERRBUF_LEN];
 
@@ -55,40 +55,46 @@ typedef struct {
 	void	*ssl;
 #endif
 
-}	UPSCONN;
+}	UPSCONN_t;
 
-const char *upscli_strerror(UPSCONN *ups);
+/* manage backward compatibility for external client application */
+/* FIXME: bug all external client apps to adapt their code */
+typedef UPSCONN_t UPSCONN;
 
-int upscli_connect(UPSCONN *ups, const char *host, int port, int flags);
+const char *upscli_strerror(UPSCONN_t *ups);
+
+int upscli_connect(UPSCONN_t *ups, const char *host, int port, int flags);
 
 /* --- functions that only use the new names --- */
 
-int upscli_get(UPSCONN *ups, unsigned int numq, const char **query, 
+int upscli_get(UPSCONN_t *ups, unsigned int numq, const char **query, 
 		unsigned int *numa, char ***answer);
 
-int upscli_list_start(UPSCONN *ups, unsigned int numq, const char **query);
+int upscli_list_start(UPSCONN_t *ups, unsigned int numq, const char **query);
 
-int upscli_list_next(UPSCONN *ups, unsigned int numq, const char **query,
+int upscli_list_next(UPSCONN_t *ups, unsigned int numq, const char **query,
 		unsigned int *numa, char ***answer);
 
-int upscli_sendline(UPSCONN *ups, const char *buf, size_t buflen);
+int upscli_sendline(UPSCONN_t *ups, const char *buf, size_t buflen);
 
-int upscli_readline(UPSCONN *ups, char *buf, size_t buflen);
+int upscli_readline(UPSCONN_t *ups, char *buf, size_t buflen);
 
 int upscli_splitname(const char *buf, char **upsname, char **hostname,
 			int *port);
 
-int upscli_sslcert(UPSCONN *ups, const char *file, const char *path, int verify);
+int upscli_splitaddr(const char *buf, char **hostname, int *port);
 
-int upscli_disconnect(UPSCONN *ups);
+int upscli_sslcert(UPSCONN_t *ups, const char *file, const char *path, int verify);
 
-/* these functions return elements from UPSCONN to avoid direct references */
+int upscli_disconnect(UPSCONN_t *ups);
 
-int upscli_fd(UPSCONN *ups);
-int upscli_upserror(UPSCONN *ups);
+/* these functions return elements from UPSCONN_t to avoid direct references */
+
+int upscli_fd(UPSCONN_t *ups);
+int upscli_upserror(UPSCONN_t *ups);
 
 /* returns 1 if SSL mode is active for this connection */
-int upscli_ssl(UPSCONN *ups);	
+int upscli_ssl(UPSCONN_t *ups);	
 
 /* upsclient error list */
 
@@ -148,6 +154,10 @@ int upscli_ssl(UPSCONN *ups);
 
 #define UPSCLI_CONN_TRYSSL	0x0001	/* try SSL, OK if not supported       */
 #define UPSCLI_CONN_REQSSL	0x0002	/* try SSL, fail if not supported     */
+#ifdef	HAVE_IPV6
+#define UPSCLI_CONN_INET	0x0004	/* IPv4 only */
+#define UPSCLI_CONN_INET6	0x0008	/* IPv6 only */
+#endif
 
 #ifdef __cplusplus
 }
