@@ -6,7 +6,7 @@
  * emes -at- geomer.de *
  * All rights reserved.*
 
- Copyright (C) 2004 Kjell Claesson <kjell.claesson-at-telia.com>
+ Copyright (C) 2004 Kjell Claesson <kjell.claesson-at-epost.tidanet.se>
  and Tore Ørpetveit <tore-at-orpetveit.net>
 
  Thanks to Tore Ørpetveit <tore-at-orpetveit.net> that sent me the
@@ -17,7 +17,9 @@
 
  Modified for USB by Wolfgang Ocker <weo@weo1.de>
  
- This program is free software; you can redistribute it and/or modify
+ ojw0000 2007Apr5 Oliver Wilcock - modified to control individual load segments (outlet.2.shutdown.return) on Powerware PW5125.
+ 
+This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
@@ -30,6 +32,90 @@
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+
+TODO List:
+
+	Extend the parsing of the Standard ID Block, to read:
+
+		Size of outlet monitoring block: (High priority)
+		To check if a outlet block is present.
+		Parse the outlet block to get the info about the
+		number of outlets and load segment state
+		(On, Off, On pending Off, Off pending On, Failed and closed, Failed and Open)
+		And the timers (Auto off delay, Auto on delay)
+		If this exist it is possible to use the
+		'Set outlet parameter command (0x97)' to alter the delay
+		settings or turn the outlet on or off with a delay (0 - 32767 seconds)
+		Also enable the outlet on off or shutdown-return commands in the driver.
+		(Check 'Communication Port List Block' down the list for more info)
+
+		Config Block Length: (High priority)
+		Give information if config block is
+		present, and how long it is, if it exist.
+		If config block exist, read the config block and setup
+		the possible config commands, and parse the 
+		'Length of the Extended Limits Configuration Block' for
+		extended configuration commands 
+
+		Statistic map Size: (Low priority)
+		May be used to se if there is a Statistic Map.
+		It holds data on the utility power quality for
+		the past month and since last reset. Number of
+		times on battery and how long. Up time and utility
+		frequency deviation. (Only larger ups'es)
+
+		Size of Alarm History Log: (Low priority)
+		See if it have any alarm history block and enable
+		command to dump it.
+
+		Size of Topology Block: (Medium priority)
+		Check if the topology block exist. Parse it for
+		some additional info. Type of ups input phases etc.
+
+		Maximum Supported Command Length: ( Med. to High priority)
+		Give info about the ups receive buffer size.
+
+		Size of Command List Block: ( Med. to High priority)
+		Tell me if the command block exist. Can use this to ask
+		for command list and set up the commands accepted by the ups.
+
+		Size of Alarm Block: ( Med. to High priority)
+		Make a smarter handling of the Active alarm's if we know the length
+		of the Active Alarm Block. Don't need the long loop to parse the
+		alarm's. Maybe use another way to set up the alarm struct in the
+		'init_alarm_map'.
+
+	Parse 'Communication Capabilities Block' ( Low priority)
+		Get info of the connected ports ID, number of baud rates,
+		command and respnse length.
+
+	Parse 'Communication Port List Block': ( Low priority)
+		This block gives info about the communication ports. Some ups'es
+		have multiple comport's, and use one port for eatch load segment.
+		In this block it is possible to get:
+		Number of ports. (In this List)
+		This Comport id (Which Comm Port is reporting this block.)
+		Comport id (Id for eatch port listed. The first comport ID=1)
+		Baudrate of the listed port.
+		Serial config.
+		Port usage:
+			What this Comm Port is being used for:
+			0 = Unknown usage, No communication occurring.
+			1 = Undefined / Unknown communication occurring
+			2 = Waiting to communicate with a UPS
+			3 = Communication established with a UPS
+			4 = Waiting to communicate with software or adapter
+			5 = Communication established software (e.g., LanSafe) 
+				or adapter (e.g., ConnectUPS)
+			6 = Communicating with a Display Device
+			7 = Multi-drop Serial channel
+			8 = Communicating with an Outlet Controller
+		Number of outlets. (Number of Outlets "assigned to" (controlled by) this Comm Port)
+		Outlet number. (Each assigned Outlet is listed (1-64))
+
+
+	Rewrite some parts of the driver, to minimise code duplication. (Like the inst commands) 
+
 */
 
 
@@ -40,7 +126,7 @@
 #include "bcmxcp_io.h"
 #include "bcmxcp.h"
 
-#define DRV_VERSION "0.11"
+#define DRV_VERSION "0.13"
 
 static int get_word(const unsigned char*);
 static long int get_long(const unsigned char*);
@@ -57,6 +143,7 @@ char *FreqTol[3] = {"+/-2%", "+/-5%", "+/-7"};
 char *ABMStatus[4] = {"Charging", "Discharging", "Floating", "Resting"};
 unsigned char AUTHOR[4] = {0xCF, 0x69, 0xE8, 0xD5};		/* Autorisation	command	*/
 int nphases = 0;
+char *cpu_name[] = {"Cont:", "Inve:", "Rect:", "Netw:", "Disp:"};
 
 /* get_word funktion from nut driver metasys.c */
 int get_word(const unsigned char *buffer)	/* return an integer reading a word in the supplied buffer */
@@ -175,7 +262,7 @@ unsigned char calc_checksum(const unsigned char *buf)
 void init_meter_map()
 {
 	/* Clean entire map */
-	memset(&bcmxcp_meter_map, 0, sizeof(BCMXCP_METER_MAP_ENTRY) * BCMXCP_METER_MAP_MAX);
+	memset(&bcmxcp_meter_map, 0, sizeof(BCMXCP_METER_MAP_ENTRY_t) * BCMXCP_METER_MAP_MAX);
 
 	/* Set all corresponding mappings NUT <-> BCM/XCP */
 	bcmxcp_meter_map[0].nut_entity = "output.L1-L2.voltage";
@@ -218,7 +305,7 @@ if (nphases == 1) {
 void init_alarm_map()
 {
 	/* Clean entire map */
-	memset(&bcmxcp_alarm_map, 0, sizeof(BCMXCP_ALARM_MAP_ENTRY) * BCMXCP_ALARM_MAP_MAX);
+	memset(&bcmxcp_alarm_map, 0, sizeof(BCMXCP_ALARM_MAP_ENTRY_t) * BCMXCP_ALARM_MAP_MAX);
 	
 	/* Set all alarm descriptions	*/
 	bcmxcp_alarm_map[BCMXCP_ALARM_INVERTER_AC_OVER_VOLTAGE].alarm_desc = "INVERTER_AC_OVER_VOLTAGE"; 
@@ -551,7 +638,8 @@ void decode_meter_map_entry(const unsigned char *entry, const unsigned char form
 
 void init_ups_alarm_map(const unsigned char *map, unsigned char len)
 {
-	unsigned int iIndex = 0, iOffset = 0, alarm = 0;
+	unsigned int iIndex = 0, iOffset = 0;
+	int alarm = 0;
 	
 	/* In case of debug - make explanation of values */
 	upsdebugx(2, "Index\tAlarm\tSupported\n");
@@ -727,23 +815,11 @@ void upsdrv_initinfo(void)
 	unsigned char answer[256];
 	char *pTmp, sValue[17];
 	int iRating = 0, iIndex = 0, res, len;
-	int voltage = 0;
+	int voltage = 0, ncpu = 0, buf;
 
 	/* Set driver version info */
 	dstate_setinfo("driver.version.internal", "%s", DRV_VERSION);
 
-	/* Get information on Phases from UPS */
-	res = command_read_sequence(PW_UPS_TOP_DATA_REQ, answer);
-	if (res <= 0)
-		fatal_with_errno("Could not communicate with the ups");
-
-	nphases = (answer[0] & 0x0F) +1;
-	dstate_setinfo("input.phases", "%d", nphases);
-
-
-	/* Init BCM/XCP <-> NUT meter map */
-	init_meter_map();
-
 	/* Init BCM/XCP alarm descriptions */
 	init_alarm_map();
 
@@ -756,20 +832,31 @@ void upsdrv_initinfo(void)
 	/* Get information on UPS from UPS */
 	res = command_read_sequence(PW_ID_BLOCK_REQ, answer);
 	if (res <= 0)
-		fatal_with_errno("Could not communicate with the ups");
+		fatal_with_errno(EXIT_FAILURE, "Could not communicate with the ups");
 
 	/* Get number of CPU's in ID block */
 	len = answer[iIndex++];
 
+	buf = len * 11;
+	pTmp = xmalloc(buf+1);
+
+	pTmp[0] = 0;
 	/* If there is one or more CPU number, get it */
 	if (len > 0) {
 		do {
-			/* Get the ups firmware. The major number is in the last byte, the minor is in the first */
-			dstate_setinfo("ups.firmware", "%02x.%02x", (unsigned char)answer[iIndex+1],
-				(unsigned char)answer[iIndex]);
+			if ((answer[iIndex] != 0x00) || (answer[iIndex+1] != 0x00)) {
+				/* Get the ups firmware. The major number is in the last byte, the minor is in the first */
+				snprintfcat(pTmp, buf+1, "%s%02x.%02x ", cpu_name[ncpu], answer[iIndex+1], answer[iIndex]);
+			}
 			iIndex += 2;
 			len--;
-		} while ((strcmp("00.00", dstate_getinfo("ups.firmware")) == 0) && len > 0);
+			ncpu++;
+
+		} while ((len > 0) && (ncpu <= 5));
+
+		dstate_setinfo("ups.firmware", "%s", pTmp);
+
+		free(pTmp);
 
 		/* Increment index to point at end of CPU bytes. */
 		iIndex += len * 2;
@@ -786,8 +873,15 @@ void upsdrv_initinfo(void)
 	}
 	dstate_setinfo("ups.power.nominal", "%d", iRating);
 
-	/* Skip	UPS' number of phases and phase angle, as NUT do not care */
-	iIndex += 2;
+	/* Get information on Phases from UPS */
+	nphases = (answer[iIndex++]);
+	dstate_setinfo("output.phases", "%d", nphases);
+
+	/* Init BCM/XCP <-> NUT meter map */
+	init_meter_map();
+
+	/* Skip	UPS' phase angle, as NUT do not care */
+	iIndex += 1;
 	
 	/* Get length of UPS description */
 	len = answer[iIndex++];
@@ -822,16 +916,16 @@ void upsdrv_initinfo(void)
 	/* Get information on UPS configuration */
 	res = command_read_sequence(PW_CONFIG_BLOC_REQ, answer);
 	if (res <= 0)
-		fatal_with_errno("Could not communicate with the ups");
+		fatal_with_errno(EXIT_FAILURE, "Could not communicate with the ups");
 
 	/* Get validation mask for status bitmap */
 	bcmxcp_status.topology_mask = answer[BCMXCP_CONFIG_BLOCK_HW_MODULES_INSTALLED_BYTE4];
 
-	/* Nominal input voltage of ups */
+	/* Nominal output voltage of ups */
 	voltage = get_word((answer + BCMXCP_CONFIG_BLOCK_NOMINAL_OUTPUT_VOLTAGE));
 	
 	if (voltage != 0)
-		dstate_setinfo("ups.voltage.nominal", "%d", voltage);
+		dstate_setinfo("output.voltage.nominal", "%d", voltage);
 
 	/* UPS serial number */
 	sValue[16] = 0;
@@ -851,7 +945,7 @@ void upsdrv_initinfo(void)
 	/* Get information on UPS extended limits */
 	res = command_read_sequence(PW_LIMIT_BLOCK_REQ, answer);
 	if (res <= 0)
-		fatal_with_errno("Could not communicate with the ups");
+		fatal_with_errno(EXIT_FAILURE, "Could not communicate with the ups");
 
 	/* Low battery warning */
 	bcmxcp_status.lowbatt = answer[BCMXCP_EXT_LIMITS_BLOCK_LOW_BATT_WARNING] * 60;
@@ -863,6 +957,8 @@ void upsdrv_initinfo(void)
 	/* Add instant commands */
 	dstate_addcmd("shutdown.return");
 	dstate_addcmd("shutdown.stayoff");
+	dstate_addcmd("outlet.1.shutdown.return"); /* ojw0000 */
+	dstate_addcmd("outlet.2.shutdown.return"); /* ojw0000 */
 	dstate_addcmd("test.battery.start");
 	upsh.instcmd = instcmd;
 	return;
@@ -940,25 +1036,27 @@ void upsdrv_updateinfo(void)
 		bcmxcp_status.alarm_low_battery = 0;
 
 		/* Set alarms	*/
-		/* alarm_init(); Alarms are not	supported by NUT */
+		alarm_init();
 
-		/* Check "On Battery"	alarm	*/
-		if (bcmxcp_alarm_map[BCMXCP_ALARM_UPS_ON_BATTERY].alarm_block_index	>= 0 &&
-				answer[bcmxcp_alarm_map[BCMXCP_ALARM_UPS_ON_BATTERY].alarm_block_index]	>	0)
-		{ 
-			bcmxcp_status.alarm_on_battery = 1;
-			/* alarm_set(ONBATTERY); */
+		/* Loop thru alarm map, get all alarms UPS is willing to offer */
+		for (iIndex = 0; iIndex < BCMXCP_ALARM_MAP_MAX; iIndex++){
+			if (bcmxcp_alarm_map[iIndex].alarm_block_index >= 0 && bcmxcp_alarm_map[iIndex].alarm_desc != NULL) {
+				if (answer[bcmxcp_alarm_map[iIndex].alarm_block_index]	> 0) {
+					alarm_set(bcmxcp_alarm_map[iIndex].alarm_desc);
+
+					if (iIndex == BCMXCP_ALARM_UPS_ON_BATTERY) {
+						bcmxcp_status.alarm_on_battery = 1;
+					}
+
+					if (iIndex == BCMXCP_ALARM_BATTERY_LOW) {
+						bcmxcp_status.alarm_low_battery = 1;
+					}
+				}
+			}
 		}
-		
-		if (bcmxcp_alarm_map[BCMXCP_ALARM_BATTERY_LOW].alarm_block_index >= 0 &&
-			answer[bcmxcp_alarm_map[BCMXCP_ALARM_BATTERY_LOW].alarm_block_index] > 0)
-		{ 
-			bcmxcp_status.alarm_low_battery = 1;
-			/* alarm_set(LOWBATTERY);	*/ 
-		}
-		
+
 		/* Confirm alarms	*/
-		/* alarm_commit(); */
+		alarm_commit();
 	}
 
 	/* Get status info from UPS */
@@ -1024,6 +1122,7 @@ void upsdrv_updateinfo(void)
 
 		status_commit();
 	} 
+
 	dstate_dataok();
 }
 
@@ -1034,6 +1133,12 @@ void upsdrv_shutdown(void)
 	
 	int res, sec;
 
+	/* Get vars from ups.conf */
+	if (getval("shutdown_delay") != NULL)
+		bcmxcp_status.shutdowndelay = atoi(getval("shutdown_delay"));
+	else
+		bcmxcp_status.shutdowndelay = 120;
+
 	/* maybe try to detect the UPS here, but try a shutdown	even if
 		 it doesn't respond at first if possible */
 	send_write_command(AUTHOR, 4);
@@ -1071,15 +1176,15 @@ void upsdrv_shutdown(void)
 			break;
 			}
 		case 0x33: {
-			fatalx("shutdown disbled by front panel");
+			fatalx(EXIT_FAILURE, "shutdown disbled by front panel");
 			break;
 			}
 		case 0x36: {
-			fatalx("Invalid parameter");
+			fatalx(EXIT_FAILURE, "Invalid parameter");
 			break;
 			}
 		default: {
-			fatalx("shutdown not supported");
+			fatalx(EXIT_FAILURE, "shutdown not supported");
 			break;
 			}
 	}
@@ -1088,11 +1193,60 @@ void upsdrv_shutdown(void)
 
 static int instcmd(const char *cmdname, const char *extra)
 {
-	unsigned char answer[5], cbuf[3];
+	unsigned char answer[5], cbuf[6];
 
 	int res, sec;
 	
 	
+	/* ojw0000 outlet power cycle for PW5125 and perhaps others */
+	if (!strcasecmp(cmdname, "outlet.1.shutdown.return")
+		|| !strcasecmp(cmdname, "outlet.2.shutdown.return")
+	) {
+		send_write_command(AUTHOR, 4);
+
+		sleep(1);	/* Need to. Have to wait at least 0,25 sec max 16 sec */
+
+		cbuf[0] = PW_LOAD_OFF_RESTART;
+		cbuf[1] = 0x03; /* outlet off in 3 seconds */
+		cbuf[2] = 0x00; /* high byte of the 2 byte time argument */
+		cbuf[3] = ( '1' == cmdname[7] ? 0x01 : 0x02); /* which outlet load segment?  Assumes '1' or '2' at position 8 of the command string. */
+
+		/* ojw00000 the following copied from command "shutdown.return" below 2007Apr5 */
+		res = command_write_sequence(cbuf, 4, answer);
+		if (res <= 0) {
+			upslogx(LOG_ERR, "Short read from UPS");
+			dstate_datastale();
+			return -1;
+		}
+	
+		sec = (256 * (unsigned char)answer[3]) + (unsigned char)answer[2];
+	
+		switch ((unsigned char) answer[0]) {
+	
+			case 0x31: {
+				upslogx(LOG_NOTICE,"Going down in %d sec", sec);
+				return STAT_INSTCMD_HANDLED;
+				break;
+				}
+			case 0x33: {
+				upslogx(LOG_NOTICE, "[%s] disbled by front panel", cmdname);
+				return STAT_INSTCMD_UNKNOWN;
+				break;
+				}
+			case 0x36: {
+			upslogx(LOG_NOTICE, "[%s] Invalid parameter", cmdname);
+				return STAT_INSTCMD_UNKNOWN;
+				break;
+				}
+			default: {
+				upslogx(LOG_NOTICE, "[%s] not supported", cmdname);
+				return STAT_INSTCMD_UNKNOWN;
+				break;
+				}
+		}
+
+	} /* ojw0000 end outlet power cycle */
+
 	if (!strcasecmp(cmdname, "shutdown.return")) {
 		send_write_command(AUTHOR, 4);
 	
@@ -1242,4 +1396,6 @@ void upsdrv_banner(void)
 {
 	printf("Network UPS Tools - BCMXCP UPS driver %s (%s)\n\n",
 		DRV_VERSION, UPS_VERSION);
+
+	experimental_driver = 1;	
 }
