@@ -143,7 +143,7 @@ static int do_broken_rat(char *buf)
 
 	usleep(25000);
 
-	ret = ser_get_buf_len(upsfd, tmp, 7, 3, 0);
+	ret = ser_get_buf_len(upsfd, (unsigned char *)tmp, 7, 3, 0);
 
 	if (ret != 7)
 		return -1;
@@ -160,7 +160,7 @@ static int do_broken_rat(char *buf)
 	if (cnt == 53)
 		cnt = 50;
 
-	ret = ser_get_buf_len(upsfd, buf, 50, cnt, 0);
+	ret = ser_get_buf_len(upsfd, (unsigned char *)buf, 50, cnt, 0);
 	buf[cnt] = 0;
 
 	return ret;
@@ -179,7 +179,7 @@ static int init_ups_data(void)
 
 	dstate_setinfo("ups.model", "%s", temp);
 
-	send_belkin_command(STATUS, VERSION, "");
+	send_belkin_command(STATUS, VERSION_CMD, "");
 	res = get_belkin_reply(temp);
 	if (res == -1)
 		return res;
@@ -206,7 +206,7 @@ static int init_ups_data(void)
 		dstate_setinfo("input.transfer.high", "%03.1f", high);
 	}
 
-	tcflush(upsfd,TCIOFLUSH);
+	ser_flush_io(upsfd);
 
 	dstate_addcmd("load.off");
 	dstate_addcmd("load.on");
@@ -284,7 +284,7 @@ static int get_belkin_reply(char *buf)
 	usleep(25000);
 
 	/* pull first 7 bytes to get data length - like ~00S004 */
-	ret = ser_get_buf_len(upsfd, tmp, 7, 3, 0);
+	ret = ser_get_buf_len(upsfd, (unsigned char *)tmp, 7, 3, 0);
 
 	if (ret != 7) {
 		ser_comm_fail("Initial read returned %d bytes", ret);
@@ -300,7 +300,7 @@ static int get_belkin_reply(char *buf)
 	/* give it time to respond to us */
 	usleep(5000 * cnt);
 
-	ret = ser_get_buf_len(upsfd, buf, cnt, 3, 0);
+	ret = ser_get_buf_len(upsfd, (unsigned char *)buf, cnt, 3, 0);
 
 	buf[cnt] = 0;
 
@@ -382,12 +382,9 @@ static int instcmd(const char *cmdname, const char *extra)
 
 static void set_serialDTR0RTS1(void)
 {
-	int dtr_bit = TIOCM_DTR;
-	int rts_bit = TIOCM_RTS;
-
 	/* set DTR to low and RTS to high */
-	ioctl(upsfd, TIOCMBIC, &dtr_bit);
-	ioctl(upsfd, TIOCMBIS, &rts_bit);
+	ser_set_dtr(upsfd, 0);
+	ser_set_rts(upsfd, 1);
 }
 
 void upsdrv_banner(void)
@@ -413,7 +410,8 @@ void upsdrv_initups(void)
 	set_serialDTR0RTS1();
 
 	sleep(1);
-  	tcflush(upsfd,TCIOFLUSH);
+
+	ser_flush_io(upsfd);
 }
 
 void upsdrv_initinfo(void)
@@ -422,10 +420,10 @@ void upsdrv_initinfo(void)
 
 	res = init_communication();
 	if (res == -1) {
-		printf("Unable to detect an Belkin Smart protocol UPS on port %s\n", 
-			device_path);
-		printf("Check the cabling, port name or model name and try again\n");
-		exit(EXIT_FAILURE);
+		fatalx(EXIT_FAILURE, 
+			"Unable to detect an Belkin Smart protocol UPS on port %s\n"
+			"Check the cabling, port name or model name and try again", device_path
+			);
 	}
 
 	dstate_setinfo("ups.mfr", "BELKIN");
