@@ -116,10 +116,11 @@ void  upsdrv_initinfo (void)
 		dstate_setinfo("ups.model", "%s LI%d", fc.name, fc.va);
 		break;
 	  default:
-		exit(EXIT_FAILURE); /* Will never get here, upsdrv_initups() will catch */
+		fatalx(EXIT_FAILURE, "Unknown model - oops!"); /* Will never get here, upsdrv_initups() will catch */
 	} 
 
-	dstate_setinfo("battery.voltage.nominal", "%05.2f", (double)fc.idealbvolts);
+	dstate_setinfo("ups.power.nominal", "%d", fc.va);
+	dstate_setinfo("ups.realpower.nominal", "%d", fc.watts);
 
 	/* Do we really need to waste time on this? */
 	/*
@@ -135,6 +136,8 @@ void  upsdrv_initinfo (void)
 		}
 	*/
 
+	dstate_setinfo("battery.voltage.nominal", "%05.2f", (double)fc.idealbvolts);
+
 	upsdebugx(1, "Best Power %s detected", dstate_getinfo("ups.model"));
 	upsdebugx(1, "Battery voltages: %5.2f nominal, %5.2f full, %5.2f low, %5.2f empty",
 		fc.idealbvolts,
@@ -170,7 +173,7 @@ static void alert_handler(char ch)
 	 * "\r\n{Inverter:     On}\r\n=>"
 	 * Try to flush the message
 	 */
-	ser_get_line(upsfd, buf, sizeof(buf), '\012', "", 0, 20);
+	ser_get_line(upsfd, buf, sizeof(buf), '\012', "", 0, 20000);
 }
 
 /* Debugging display from kermit:
@@ -181,18 +184,22 @@ time^M^M^JFeb 20, 22:13:32^M^J^M^J=>id^M^JUnit ID "ME3.1K12345"^M^J^M^J=>
 static int execute(const char *cmd, char *result, int resultsize)
 {
 	int ret;
-	char ch, buf[256];
+	char buf[256];
+	unsigned char ch;
 
 	/* Check for the Inverter status alarm if pending :
 	 * "\r\n{Inverter:     On}\r\n=>"
 	 */
 	ser_get_line_alert(upsfd, buf, sizeof(buf), '\012', "",
-		POLL_ALERT, alert_handler, 0, 20);
+		POLL_ALERT, alert_handler, 0, 20000);
 
 	ser_send(upsfd, cmd);
 
+	/* Give the UPS some time to chew on what we just sent */
+	usleep(50000);
+
 	/* delete command echo up to \012 but no further */
-	for (ch = '\0'; ch != '\012'; ser_get_char(upsfd, &ch, 0, 10));
+	for (ch = '\0'; ch != '\012'; ser_get_char(upsfd, &ch, 0, 10000));
 
 	/* get command response	*/
 	ret = ser_get_line(upsfd, result, resultsize, '\015', "\012", 3, 0);
@@ -278,7 +285,7 @@ void upsdrv_updateinfo(void)
 			ampsout = ((double)bcd2i(&fstring[36], 4) / 10.0);
 
 			/* Volt-amps out.  int	*/
-			vaout = ((double)bcd2i(&fstring[40], 6) / 10);
+			vaout = bcd2i(&fstring[40], 6);
 
 			/* Line status.	 Bitmask */
 			linestat = bcd2i(&fstring[72], 2);
@@ -309,8 +316,7 @@ void upsdrv_updateinfo(void)
 			}
 			break;
 		  default: /* Will never happen, caught in upsdrv_initups() */
-			upsdebugx(1, "Uknown model in upsdrv_updateinfo()");
-			exit(EXIT_FAILURE);
+			fatalx(EXIT_FAILURE, "Unknown model in upsdrv_updateinfo()");
 		}
 
 		/* Compute battery percent left based on battery voltages. */
@@ -387,6 +393,8 @@ void upsdrv_updateinfo(void)
 		dstate_setinfo("battery.voltage", "%02.1f", vbatt);
 		dstate_setinfo("battery.runtime", "%d", btimeleft);
 		dstate_setinfo("ups.load", "%02.1f", loadpercent);
+		if (vaout)
+			dstate_setinfo("ups.power", "%d", vaout);
 		if (upstemp)
 			dstate_setinfo("ups.temperature", "%05.1f", (double)upstemp);
 
@@ -413,8 +421,7 @@ static void ups_sync(void)
 	if (execute("time\r", buf, sizeof(buf)) > 0) {
 		upsdebugx(1, "UPS Time: %s", buf);
 	} else {
-		upsdebugx(1, "Error connecting to UPS.");
-		exit(EXIT_FAILURE);
+		fatalx(EXIT_FAILURE, "Error connecting to UPS.");
 	}
 }
 
@@ -439,6 +446,8 @@ void upsdrv_banner(void)
 {
 	printf("Network UPS Tools - Best Ferrups/Fortress %s (%s)\n",
 		DRV_VERSION, UPS_VERSION);
+
+	experimental_driver = 1;
 }
 
 static void sync_serial(void) {
@@ -464,7 +473,7 @@ static void setup_serial(void)
 	struct termios tio;
 			 
 	if (tcgetattr(upsfd, &tio) == -1)
-		fatal_with_errno("tcgetattr");
+		fatal_with_errno(EXIT_FAILURE, "tcgetattr");
 				 
 	tio.c_iflag = IXON | IXOFF;
 	tio.c_oflag = 0;
@@ -481,7 +490,7 @@ static void setup_serial(void)
 #endif
 
 	if (tcsetattr(upsfd, TCSANOW, &tio) == -1)
-		fatal_with_errno("tcsetattr");
+		fatal_with_errno(EXIT_FAILURE, "tcsetattr");
 /* end code stolen from bestups.c */
 
 	sync_serial();
@@ -573,8 +582,7 @@ void upsdrv_init_nofc()
 	} 
 
 	if (fc.model == UNKNOWN) {
-		upsdebugx(1, "Unknown model %s in upsdrv_init_nofc()", rstring);
-		exit(EXIT_FAILURE);
+		fatalx(EXIT_FAILURE, "Unknown model %s in upsdrv_init_nofc()", rstring);
 	}
 
 	switch(fc.model) {
@@ -615,13 +623,11 @@ void upsdrv_init_nofc()
 		}
 		fc.idealbvolts = ((fc.fullvolts - fc.emptyvolts) * 0.7) + fc.emptyvolts;
 		if (fc.va < 1.0) {
-			upsdebugx(1, "Error determining Ferrups UPS rating.");
-			exit(EXIT_FAILURE);
+			fatalx(EXIT_FAILURE, "Error determining Ferrups UPS rating.");
 		}
 		break;
 	  default:
-		upsdebugx(1, "Unknown model %s in upsdrv_init_nofc()", rstring);
-		exit(EXIT_FAILURE);
+		fatalx(EXIT_FAILURE, "Unknown model %s in upsdrv_init_nofc()", rstring);
 		break;
 	}
 	fc.valid = 1;
@@ -652,17 +658,14 @@ void upsdrv_init_fc(const char *fcstring)
 
 	/* Obtain Model */
 	if (memcmp(fcstring, "$", 1)) {
-		upsdebugx(1, "Bad response from formatconfig command in upsdrv_init_fc()");
-		exit(EXIT_FAILURE);
+		fatalx(EXIT_FAILURE, "Bad response from formatconfig command in upsdrv_init_fc()");
 	}
 	if (memcmp(fcstring+3, "00", 2) == 0) {
-		upsdebugx(1, "ups type unknown in upsdrv_init_fc()");
-		exit(EXIT_FAILURE);
+		fatalx(EXIT_FAILURE, "UPS type unknown in upsdrv_init_fc()");
 	}
 
 	if (memcmp(fcstring+3, "01", 2) == 0) {
-		upsdebugx(1, "Best Patriot ups not supported");
-		exit(EXIT_FAILURE);
+		fatalx(EXIT_FAILURE, "Best Patriot UPS not supported");
 	}
 	else if (memcmp(fcstring+3, "02", 2) == 0) {
 		snprintf(fc.name, sizeof(fc.name), "%s", "FortressII");
@@ -716,9 +719,7 @@ void upsdrv_init_fc(const char *fcstring)
 		fc.idealbvolts = ((fc.fullvolts - fc.emptyvolts) * 0.7) + fc.emptyvolts;
 		break;
 	  default:
-		upsdebugx(1, "Unknown model %s in upsdrv_init_fc()", tmp);
-		exit(EXIT_FAILURE);
-		break;
+		fatalx(EXIT_FAILURE, "Unknown model %s in upsdrv_init_fc()", tmp);
 	}
 	fc.valid = 1;
 }
@@ -732,13 +733,10 @@ void upsdrv_initups ()
 	setup_serial();
 	ups_sync();
 
-	experimental_driver = 1;
-
 	inverter_status = 0;
 	fc.model = UNKNOWN;
 	if (execute("f\r", rstring, sizeof(rstring)) < 0 ) {
-		upsdebugx(1, "Failed format request in upsdrc_initups()");
-		exit(EXIT_FAILURE);
+		fatalx(EXIT_FAILURE, "Failed format request in upsdrc_initups()");
 	}
 
 	execute("fc\r", rstring, sizeof(rstring));
