@@ -22,8 +22,6 @@
 #include "serial.h"
 #include "cyberpower.h"
 
-#include <sys/ioctl.h>
-
 #include "timehead.h"
 
 /* window for repeating dangerous command (shutdown.stayoff) */
@@ -203,7 +201,7 @@ static int confirm_write(const unsigned char *buf, size_t buflen)
 	}
 
 	/* don't try to read back the \r */
-	ret = ser_get_buf_len(upsfd, verify, buflen - 1, 5, 0);
+	ret = ser_get_buf_len(upsfd, (unsigned char *)verify, buflen - 1, 5, 0);
 
 	if ((ret < 1) || (ret < ((int) buflen - 1))) {
 		upsdebugx(1, "confirm_write: ret=%d, needed %d",
@@ -234,7 +232,7 @@ static int ups_on_line(void)
 	/* give it a chance to reply completely */
 	usleep(100000);
 
-	ret = ser_get_buf_len(upsfd, buf, 14, 3, 0);
+	ret = ser_get_buf_len(upsfd, (unsigned char *)buf, 14, 3, 0);
 
 	if (ret < 14) {
 		upslogx(LOG_ERR, "Status read failed: assuming on battery");
@@ -260,7 +258,7 @@ void upsdrv_shutdown(void)
 		fflush(stdout);
 
 		ser_send_char(upsfd, 13);
-		ret = ser_get_buf_len(upsfd, buf, 1, 5, 0);
+		ret = ser_get_buf_len(upsfd, (unsigned char *)buf, 1, 5, 0);
 
 		if (ret > 0) {
 			printf("OK\n");
@@ -334,7 +332,7 @@ void upsdrv_updateinfo(void)
 	/* give it a chance to reply completely */
 	usleep(100000);
 
-	ret = ser_get_buf_len(upsfd, buf, 14, 3, 0);
+	ret = ser_get_buf_len(upsfd, (unsigned char *)buf, 14, 3, 0);
 
 	if (ret < 14) {
 		ser_comm_fail("Short read from UPS");
@@ -415,7 +413,7 @@ static int detect_hardware(void)
 	char	buf[SMALLBUF];
 
 	if (!get_ident(buf, sizeof(buf)))
-		fatalx("Unable to get initial hardware info string");
+		fatalx(EXIT_FAILURE, "Unable to get initial hardware info string");
 
 	if (buf[0] != '.') {
 		upslogx(LOG_ERR, "Invalid start model string 0x%02x", 
@@ -533,24 +531,21 @@ void upsdrv_makevartable(void)
 /* prep the serial port */
 void upsdrv_initups(void)
 {
-	int dtr_bit = TIOCM_DTR;
-	int rts_bit = TIOCM_RTS;
-
 	upsfd = ser_open(device_path);
 	ser_set_speed(upsfd, device_path, B1200);
 
 	/* dtr high, rts high */
-	ioctl(upsfd, TIOCMBIS, &rts_bit);
-	ioctl(upsfd, TIOCMBIS, &dtr_bit);
+	ser_set_dtr(upsfd, 1);
+	ser_set_rts(upsfd, 1);
 }
 
 void upsdrv_initinfo(void)
 {
 	if (detect_hardware() == -1) {
-		printf("Unable to detect a CyberPower UPS on port %s\n", 
-			device_path);
-		printf("Check the cabling, port name or model name and try again\n");
-		exit(EXIT_FAILURE);
+		fatalx(EXIT_FAILURE, 
+			"Unable to detect a CyberPower UPS on port %s\n"
+			"Check the cabling, port name or model name and try again", device_path
+		);
 	}
 
 	dstate_setinfo("ups.mfr", "CyberPower");
