@@ -35,7 +35,7 @@
 	static	struct	conn_t	*connhead = NULL;
 	static	struct	cmdlist_t *cmdhead = NULL;
 	static	char	*sockfn = NULL;
-	static	char	status_buf[ST_MAX_VALUE_LEN], 
+	static	char	status_buf[ST_MAX_VALUE_LEN],
 			alarm_buf[ST_MAX_VALUE_LEN];
 
 	struct	ups_handler	upsh;
@@ -57,7 +57,7 @@ static void sock_fail(const char *fn)
 	user = getpwuid(getuid());
 
 	if (!user)
-		fatal_with_errno("getpwuid");
+		fatal_with_errno(EXIT_FAILURE, "getpwuid");
 
 	/* deal with some common problems */
 	switch (errno) {
@@ -90,7 +90,7 @@ static void sock_fail(const char *fn)
 	 */
 
 	printf("\n");
-	fatalx("Exiting.");
+	fatalx(EXIT_FAILURE, "Exiting.");
 }
 
 static int open_sock(const char *fn)
@@ -101,7 +101,7 @@ static int open_sock(const char *fn)
 	fd = socket(AF_UNIX, SOCK_STREAM, 0);
 
 	if (fd < 0)
-		fatal_with_errno("Can't create a unix domain socket");
+		fatal_with_errno(EXIT_FAILURE, "Can't create a unix domain socket");
 
 	/* keep this around for the unlink() when exiting */
 	sockfn = xstrdup(fn);
@@ -122,12 +122,12 @@ static int open_sock(const char *fn)
 	ret = chmod(sockfn, 0660);
 
 	if (ret < 0)
-		fatal_with_errno("chmod(%s, 0660) failed", sockfn);
+		fatal_with_errno(EXIT_FAILURE, "chmod(%s, 0660) failed", sockfn);
 
 	ret = listen(fd, DS_LISTEN_BACKLOG);
 
 	if (ret < 0)
-		fatal_with_errno("listen(%d, %d) failed", fd, DS_LISTEN_BACKLOG);
+		fatal_with_errno(EXIT_FAILURE, "listen(%d, %d) failed", fd, DS_LISTEN_BACKLOG);
 
 	return fd;
 }
@@ -364,11 +364,15 @@ static int sock_arg(struct conn_t *conn, int numarg, char **arg)
 	if (numarg < 2)
 		return 0;
 
-	/* INSTCMD <cmdname> */			/* future: extra args */
+	/* INSTCMD <cmdname> [<value>]*/
 	if (!strcasecmp(arg[0], "INSTCMD")) {
 
 		/* try the new handler first if present */
 		if (upsh.instcmd) {
+			if (numarg > 2) {
+				upsh.instcmd(arg[1], arg[2]);
+				return 1;
+			}
 			upsh.instcmd(arg[1], NULL);
 			return 1;
 		}
@@ -755,10 +759,6 @@ int dstate_is_stale(void)
 void status_init(void)
 {
 	memset(&status_buf, '\0', sizeof(status_buf));
-
-	/* this is always first */
-	if (alarm_active)
-		snprintf(status_buf, sizeof(status_buf), "ALARM");
 }
 
 /* add a status element */
@@ -774,7 +774,10 @@ void status_set(const char *buf)
 /* write the status_buf into the externally visible dstate storage */
 void status_commit(void)
 {
-	dstate_setinfo("ups.status", "%s", status_buf);
+	if (alarm_active)
+		dstate_setinfo("ups.status", "ALARM %s", status_buf);
+	else
+		dstate_setinfo("ups.status", "%s", status_buf);
 }
 
 /* similar handlers for ups.alarm */
@@ -782,7 +785,6 @@ void status_commit(void)
 void alarm_init(void)
 {
 	memset(&alarm_buf, '\0', sizeof(alarm_buf));
-	alarm_active = 0;
 }
 
 void alarm_set(const char *buf)
@@ -791,46 +793,16 @@ void alarm_set(const char *buf)
 		snprintfcat(alarm_buf, sizeof(alarm_buf), " %s", buf);
 	else
 		snprintfcat(alarm_buf, sizeof(alarm_buf), "%s", buf);
-
-	alarm_active = 1;
 }
 
 /* write the status_buf into the info array */
 void alarm_commit(void)
 {
-	const	char	*statval, *alptr;
-
-	dstate_setinfo("ups.alarm", "%s", alarm_buf);
-
-	statval = dstate_getinfo("ups.status");
-
-	if (!statval) {
-		upslogx(LOG_ERR, "alarm_commit: ups.status isn't defined");
-		return;
+	if (strlen(alarm_buf) != 0) {
+		dstate_setinfo("ups.alarm", "%s", alarm_buf);
+		alarm_active = 1;
+	} else {
+		dstate_delinfo("ups.alarm");
+		alarm_active = 0;
 	}
-
-	alptr = strstr(statval, "ALARM");
-
-	if (!alarm_active) {		/* no alarm is active */
-
-		if (!alptr)		/* and it's not in ups.status */
-			return;		/* so we're done */
-
-		/* the alarm cleared - remove it from ups.status */
-
-		if (strlen(statval) <= 7)
-			dstate_setinfo("ups.status", "%s", "");
-		else
-			dstate_setinfo("ups.status", "%s", &statval[6]);
-
-		return;
-	}
-
-	/* at this point, an alarm is active */
-
-	if (alptr)
-		return;		/* already in ups.status */
-
-	/* add to ups.status */
-	dstate_setinfo("ups.status", "ALARM %s", statval);
 }
