@@ -17,7 +17,7 @@
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
-#define DRV_VERSION "1.32"
+#define DRV_VERSION "1.33"
 
 #include <sys/ioctl.h>
 
@@ -28,116 +28,111 @@
 
 	static	int	upstype = -1;
 
-static void parse_output_signals(const char *value, int *line) 
+static void parse_output_signals(const char *value, int *line)
 { 
-	/* parse signals the serial port can output */ 
+	/* parse signals the serial port can output */
  
-	*line = 0; 
+	*line = 0;
  
-	if (strstr(value, "DTR")) 
-		if (!strstr(value, "-DTR")) 
-			*line |= TIOCM_DTR; 
+	if (strstr(value, "DTR") && !strstr(value, "-DTR"))
+		*line |= TIOCM_DTR;
  
-	if (strstr(value, "RTS")) 
-		if (!strstr(value, "-RTS")) 
-			*line |= TIOCM_RTS; 
+	if (strstr(value, "RTS") && !strstr(value, "-RTS"))
+		*line |= TIOCM_RTS;
  
-	if (strstr(value, "ST")) 
-		*line |= TIOCM_ST; 
+	if (strstr(value, "ST"))
+		*line |= TIOCM_ST;
 
 	if (strstr(value, "CTS"))
-		fatalx("Can't override output with CTS (not an output)");
+		fatalx(EXIT_FAILURE, "Can't override output with CTS (not an output)");
 
 	if (strstr(value, "DCD"))
-		fatalx("Can't override output with DCD (not an output)");
+		fatalx(EXIT_FAILURE, "Can't override output with DCD (not an output)");
 
 	if (strstr(value, "RNG"))
-		fatalx("Can't override output with RNG (not an output)");
+		fatalx(EXIT_FAILURE, "Can't override output with RNG (not an output)");
 } 
  
-static void parse_input_signals(const char *value, int *line, int *val) 
+static void parse_input_signals(const char *value, int *line, int *val)
 { 
-	/* parse signals the serial port can input */ 
+	/* parse signals the serial port can input */
  
-	*line = 0; 
-	*val = 0; 
+	*line = 0;
+	*val = 0;
  
-	if (strstr(value, "CTS")) {
-		*line |= TIOCM_CTS; 
-		if (!strstr(value, "-CTS")) 
-			*val |= TIOCM_CTS; 
+	if (strstr(value, "CTS"))
+	{
+		*line |= TIOCM_CTS;
+
+		if (!strstr(value, "-CTS"))
+			*val |= TIOCM_CTS;
  	}
 
-	if (strstr(value, "DCD")) {
-		*line |= TIOCM_CD; 
-		if (!strstr(value, "-DCD")) 
-			*val |= TIOCM_CD; 
+	if (strstr(value, "DCD"))
+	{
+		*line |= TIOCM_CD;
+
+		if (!strstr(value, "-DCD"))
+			*val |= TIOCM_CD;
  	}
 
-	if (strstr(value, "RNG")) {
+	if (strstr(value, "RNG"))
+	{
 		*line |= TIOCM_RNG; 
-		if (!strstr(value, "-RNG")) 
-			*val |= TIOCM_RNG; 
+
+		if (!strstr(value, "-RNG"))
+			*val |= TIOCM_RNG;
 	}
 
 	if (strstr(value, "DTR"))
-		fatalx("Can't override input with DTR (not an input)");
+		fatalx(EXIT_FAILURE, "Can't override input with DTR (not an input)");
 
 	if (strstr(value, "RTS"))
-		fatalx("Can't override input with RTS (not an input)");
+		fatalx(EXIT_FAILURE, "Can't override input with RTS (not an input)");
 
 	if (strstr(value, "ST"))
-		fatalx("Can't override input with ST (not an input)");
+		fatalx(EXIT_FAILURE, "Can't override input with ST (not an input)");
 }
 
 void upsdrv_initinfo(void)
 {
-	/* setup the basics */
-
-	if (getval("mfr"))
-		dstate_setinfo("ups.mfr", "%s", getval("mfr"));
-	else
-		dstate_setinfo("ups.mfr", "%s", upstab[upstype].mfr);
-
-	if (getval("model"))
-		dstate_setinfo("ups.model", "%s", getval("model"));
-	else
-		dstate_setinfo("ups.model", "%s", upstab[upstype].model);
-
-	if (getval("serial"))
-		dstate_setinfo("ups.serial", "%s", getval("serial"));
+	char	*v;
 
 	dstate_setinfo("driver.version.internal", "%s", DRV_VERSION);
 
+	/* setup the basics */
+
+	dstate_setinfo("ups.mfr", "%s", ((v = getval("mfr")) != NULL) ? v : upstab[upstype].mfr);
+	dstate_setinfo("ups.model", "%s", ((v = getval("model")) != NULL) ? v : upstab[upstype].model);
+
+	if ((v = getval("serial")) != NULL)
+		dstate_setinfo("ups.serial", "%s", v);
+
 	/* see if the user wants to override the signal definitions */
 
-	if (getval("CP")) {
-		parse_output_signals(getval("CP"), &upstab[upstype].line_norm); 
-		upsdebugx(2, "parse_output_signals: CP overriden with %s\n",
-			getval("CP"));	
+	if ((v = getval("CP")) != NULL)
+	{
+		parse_output_signals(v, &upstab[upstype].line_norm);
+		upsdebugx(2, "parse_output_signals: CP overriden with %s\n", v);
 	}
-	
-	if (getval("OL")) {
-		parse_input_signals(getval("OL"), &upstab[upstype].line_ol, 
-			&upstab[upstype].val_ol); 
-		upsdebugx(2, "parse_input_signals: OL overriden with %s\n",
-			getval("OL"));	
+
+	if ((v = getval("OL")) != NULL)
+	{
+		parse_input_signals(v, &upstab[upstype].line_ol, &upstab[upstype].val_ol);
+		upsdebugx(2, "parse_input_signals: OL overriden with %s\n", v);
 	}
-	
-	if (getval("LB")) {
-		parse_input_signals(getval("LB"), &upstab[upstype].line_bl, 
-			&upstab[upstype].val_bl); 
- 		upsdebugx(2, "parse_input_signals: LB overriden with %s\n",
-			getval("LB"));
+
+	if ((v = getval("LB")) != NULL)
+	{
+		parse_input_signals(v, &upstab[upstype].line_bl, &upstab[upstype].val_bl);
+ 		upsdebugx(2, "parse_input_signals: LB overriden with %s\n", v);
 	}
-	
-	if (getval("SD")) {
-		parse_output_signals(getval("SD"), &upstab[upstype].line_sd); 
-		upsdebugx(2, "parse_output_signals: SD overriden with %s\n",
-			getval("SD"));
+
+	if ((v = getval("SD")) != NULL)
+	{
+		parse_output_signals(v, &upstab[upstype].line_sd);
+		upsdebugx(2, "parse_output_signals: SD overriden with %s\n", v);
 	}
-	
-	dstate_dataok();
 }
 
 /* normal idle loop - keep up with the current state of the UPS */
@@ -149,6 +144,7 @@ void upsdrv_updateinfo(void)
 
 	if (ret != 0) {
 		upslog_with_errno(LOG_INFO, "ioctl failed");
+		ser_comm_fail("Status read failed");
 		dstate_datastale();
 		return;
 	}
@@ -167,6 +163,10 @@ void upsdrv_updateinfo(void)
 		status_set("OB");	/* on battery */
 
 	status_commit();
+
+	upsdebugx(5, "ups.status: %s %s\n", ol ? "OL" : "OB", bl ? "BL" : "");
+
+	ser_comm_good();
 	dstate_dataok();
 }
 
@@ -187,22 +187,22 @@ static void set_ups_type(void)
 	int	i;
 
 	if (!getval("upstype"))
-		fatalx("No upstype set - see help text / man page!\n");
+		fatalx(EXIT_FAILURE, "No upstype set - see help text / man page!");
 	
 	upstype = atoi(getval("upstype"));
 
-	for (i = 0; upstab[i].mfr != NULL; i++) {
+	for (i = 0; upstab[i].mfr != NULL; i++)
+	{
 		if (upstype == i) {
-			printf("UPS type: %s\n", upstab[i].desc);
+			upslogx(LOG_INFO, "UPS type: %s\n", upstab[i].desc);
 			return;
 		}
 	}
 
-	printf("\nFatal error: unknown UPS type number\n");
 	listtypes();
 
-	exit(EXIT_FAILURE);
-}			
+	fatalx(EXIT_FAILURE, "\nFatal error: unknown UPS type number");
+}
 
 /* power down the attached load immediately */
 void upsdrv_shutdown(void)
@@ -210,30 +210,31 @@ void upsdrv_shutdown(void)
 	int	flags, ret;
 
 	if (upstype == -1)
-		fatalx("No upstype set - see help text / man page!");
+		fatalx(EXIT_FAILURE, "No upstype set - see help text / man page!");
 
 	flags = upstab[upstype].line_sd;
 
 	if (flags == -1)
-		fatalx("No shutdown command defined for this model!");
+		fatalx(EXIT_FAILURE, "No shutdown command defined for this model!");
 
 	if (flags == TIOCM_ST) {
 
 #ifndef HAVE_TCSENDBREAK
-		fatalx("Need to send a BREAK, but don't have tcsendbreak!");
+		fatalx(EXIT_FAILURE, "Need to send a BREAK, but don't have tcsendbreak!");
 #endif
 
 		ret = tcsendbreak(upsfd, 4901);
 
 		if (ret != 0)
-			fatal_with_errno("tcsendbreak");
+			fatal_with_errno(EXIT_FAILURE, "tcsendbreak");
 
 		return;
 	}
 
 	ret = ioctl(upsfd, TIOCMSET, &flags);
+
 	if (ret != 0)
-		fatal_with_errno("ioctl TIOCMSET");
+		fatal_with_errno(EXIT_FAILURE, "ioctl TIOCMSET");
 
 	if (getval("sdtime")) {
 		int	sdtime;
@@ -254,7 +255,7 @@ void upsdrv_help(void)
 
 void upsdrv_banner(void)
 {
-	printf("Network UPS Tools - Generic UPS driver %s (%s)\n", 
+	printf("Network UPS Tools - Generic UPS driver %s (%s)\n",
 		DRV_VERSION, UPS_VERSION);
 }
 
@@ -264,10 +265,10 @@ void upsdrv_makevartable(void)
 	addvar(VAR_VALUE, "mfr", "Override manufacturer name");
 	addvar(VAR_VALUE, "model", "Override model name");
 	addvar(VAR_VALUE, "serial", "Specify the serial number");
-	addvar(VAR_VALUE, "CP", "Override cable power setting"); 
-	addvar(VAR_VALUE, "OL", "Override on line signal"); 
-	addvar(VAR_VALUE, "LB", "Override low battery signal"); 
-	addvar(VAR_VALUE, "SD", "Override shutdown setting"); 
+	addvar(VAR_VALUE, "CP", "Override cable power setting");
+	addvar(VAR_VALUE, "OL", "Override on line signal");
+	addvar(VAR_VALUE, "LB", "Override low battery signal");
+	addvar(VAR_VALUE, "SD", "Override shutdown setting");
 	addvar(VAR_VALUE, "sdtime", "Hold time for shutdown value (seconds)");
 }
 
@@ -278,11 +279,10 @@ void upsdrv_initups(void)
 	upsfd = ser_open(device_path);
 
 	if (ioctl(upsfd, TIOCMSET, &upstab[upstype].line_norm))
-		fatal_with_errno("ioctl TIOCMSET");
+		fatal_with_errno(EXIT_FAILURE, "ioctl TIOCMSET");
 }
 
 void upsdrv_cleanup(void)
 {
 	ser_close(upsfd, device_path);
 }
-
