@@ -27,6 +27,8 @@
 #include "config.h"
 #include "hidparser.h"
 
+#include "nut_stdint.h" /* for int8_t, int16_t, int32_t */
+
 /* to be implemented for DEBUG purpose */
 /* previously: #define ERROR(x) if(x) __asm { int 3 }; */
 #define ERROR(x)
@@ -38,25 +40,25 @@ static const char ItemSize[4]={0,1,2,4};
  * -------------------------------------------------------------------------- */
 typedef struct
 {
-	u_char   ReportDesc[REPORT_DSC_SIZE];	/* Store Report Descriptor */
-	u_short  ReportDescSize;					/* Size of Report Descriptor */
-	u_short  Pos;							/* Store current pos in descriptor */
-	u_char   Item;							/* Store current Item */
-	long    Value;							/* Store current Value */
+	u_char   *ReportDesc;			/* Report Descriptor */
+	u_short  ReportDescSize;		/* Size of Report Descriptor */
+	u_short  Pos;				/* Store current pos in descriptor */
+	u_char   Item;				/* Store current Item */
+	long    Value;				/* Store current Value */
 
-	HIDData Data;							/* Store current environment */
+	HIDData_t Data;				/* Store current environment */
 
 	u_char   OffsetTab[MAX_REPORT][4];	/* Store ID, Type, offset & timestamp of report	*/
-	u_char   ReportCount;					/* Store Report Count */
-	u_char   Count;							/* Store local report count */
+	u_char   ReportCount;			/* Store Report Count */
+	u_char   Count;				/* Store local report count */
 
-	u_short  UPage;							/* Global UPage */
-	HIDNode UsageTab[USAGE_TAB_SIZE];	/* Usage stack */
-	u_char   UsageSize;						/* Design number of usage used */
+	u_short  UPage;				/* Global UPage */
+	HIDNode_t UsageTab[USAGE_TAB_SIZE];	/* Usage stack */
+	u_char   UsageSize;			/* Design number of usage used */
 
-	u_char   nObject;						/* Count Objects in Report Descriptor */
-	u_char   nReport;						/* Count Reports in Report Descriptor */
-} HIDParser;
+	u_char   nObject;			/* Count Objects in Report Descriptor */
+	u_char   nReport;			/* Count Reports in Report Descriptor */
+} HIDParser_t;
 
 /* return 1 + the position of the leftmost "1" bit of an int, or 0 if
    none. */
@@ -75,25 +77,6 @@ static inline unsigned int hibit(unsigned int x) {
   return res;
 }
 
-/*
- * ResetParser
- * Reset HIDParser structure for new parsing
- * Keep Report descriptor data
- * -------------------------------------------------------------------------- */
-static void ResetParser(HIDParser* pParser)
-{
-  pParser->Pos=0;
-  pParser->Count=0;
-  pParser->nObject=0;
-  pParser->nReport=0;
-
-  pParser->UsageSize=0;
-  memset(pParser->UsageTab,0,sizeof(pParser->UsageTab));
-
-  memset(pParser->OffsetTab,0,sizeof(pParser->OffsetTab));
-  memset(&pParser->Data,0,sizeof(pParser->Data));
-}
-
 /* Note: The USB HID specification states that Local items do not
    carry over to the next Main item (version 1.11, section
    6.2.2.8). Therefore the local state must be reset after each main
@@ -103,7 +86,7 @@ static void ResetParser(HIDParser* pParser)
    single control, so resetting the local state is important. */
 /* Also note: UsageTab[0] is used as the usage of the next control,
    even if UsageSize=0. Therefore, this must be initialized */
-static void ResetLocalState(HIDParser* pParser)
+static void ResetLocalState(HIDParser_t* pParser)
 {
   pParser->UsageSize = 0;
   memset(pParser->UsageTab,0,sizeof(pParser->UsageTab));
@@ -115,7 +98,7 @@ static void ResetLocalState(HIDParser* pParser)
  * Return pointer on current offset value for Report designed by 
  * ReportID/ReportType
  * -------------------------------------------------------------------------- */
-static u_char* GetReportOffset(HIDParser* pParser, 
+static u_char* GetReportOffset(HIDParser_t* pParser, 
                        const u_char ReportID, 
                        const u_char ReportType)
 {
@@ -145,23 +128,28 @@ static u_char* GetReportOffset(HIDParser* pParser,
  * -------------------------------------------------------------------------- */
 static long FormatValue(long Value, u_char Size)
 {
-  if(Size==1) 
-    Value=(long)(char)Value;
-  else if(Size==2) 
-    Value=(long)(short)Value;
-  return Value;
+	switch(Size)
+	{
+	case 1:
+		return (long)(int8_t)Value;
+	case 2:
+		return (long)(int16_t)Value;
+	case 4:
+		return (long)(int32_t)Value;
+	default:
+		return Value;
+	}
 }
 
 /*
- * HIDParse(HIDParser* pParser, HIDData* pData)
+ * HIDParse(HIDParser_t* pParser, HIDData_t* pData)
  *
  * Analyse Report descriptor stored in HIDParser struct and store local and
  * global context. 
- * Use ResetParser() to init HIDParser structure before beginning.
  * Return in pData the last object found.
  * Return TRUE when there is other Item to parse.
  * -------------------------------------------------------------------------- */
-static int HIDParse(HIDParser* pParser, HIDData* pData)
+static int HIDParse(HIDParser_t* pParser, HIDData_t* pData)
 {
   int Found=0;
 
@@ -293,7 +281,7 @@ static int HIDParse(HIDParser* pParser, HIDData* pData)
     
         /* Get Object in pData */
         /* -------------------------------------------------------------------------- */
-        memcpy(pData, &pParser->Data, sizeof(HIDData));
+        memcpy(pData, &pParser->Data, sizeof(HIDData_t));
         /* -------------------------------------------------------------------------- */
 
         /* Increment Report Offset */
@@ -379,9 +367,9 @@ static int HIDParse(HIDParser* pParser, HIDData* pData)
  * Get pData characteristics from pData->Path or from pData->ReportID/Offset
  * Return TRUE if object was found
  * -------------------------------------------------------------------------- */
-int FindObject(HIDDesc *pDesc, HIDData* pData)
+int FindObject(HIDDesc_t *pDesc, HIDData_t* pData)
 {
-  HIDData *pFoundData;
+  HIDData_t *pFoundData;
   int i;
 
   for (i=0; i<pDesc->nitems; i++)
@@ -389,9 +377,9 @@ int FindObject(HIDDesc *pDesc, HIDData* pData)
     pFoundData = &pDesc->item[i];
     if(pData->Path.Size>0 && 
       pFoundData->Type==pData->Type &&
-      memcmp(pFoundData->Path.Node, pData->Path.Node, (pData->Path.Size)*sizeof(HIDNode))==0)
+      memcmp(pFoundData->Path.Node, pData->Path.Node, (pData->Path.Size)*sizeof(HIDNode_t))==0)
     {
-      memcpy(pData, pFoundData, sizeof(HIDData));
+      memcpy(pData, pFoundData, sizeof(HIDData_t));
       return 1;
     }
     /* Found by ReportID/Offset */
@@ -399,7 +387,7 @@ int FindObject(HIDDesc *pDesc, HIDData* pData)
       pFoundData->Type==pData->Type &&
       pFoundData->Offset==pData->Offset)
     {
-      memcpy(pData, pFoundData, sizeof(HIDData));
+      memcpy(pData, pFoundData, sizeof(HIDData_t));
       return 1;
     }
   }
@@ -410,16 +398,16 @@ int FindObject(HIDDesc *pDesc, HIDData* pData)
  * FindObject_with_Path
  * Get pData item with given Path and Type. Return NULL if not found. 
  * -------------------------------------------------------------------------- */
-HIDData *FindObject_with_Path(HIDDesc *pDesc, HIDPath *Path, u_char Type)
+HIDData_t *FindObject_with_Path(HIDDesc_t *pDesc, HIDPath_t *Path, u_char Type)
 {
-  HIDData *pData;
+  HIDData_t *pData;
   int i;
 
   for (i=0; i<pDesc->nitems; i++)
   {
     pData = &pDesc->item[i];
     if (pData->Type == Type &&
-      memcmp(pData->Path.Node, Path->Node, (Path->Size)*sizeof(HIDNode)) == 0)
+      memcmp(pData->Path.Node, Path->Node, (Path->Size)*sizeof(HIDNode_t)) == 0)
     {
       return pData;
     }
@@ -432,9 +420,9 @@ HIDData *FindObject_with_Path(HIDDesc *pDesc, HIDPath *Path, u_char Type)
  * Get pData item with given ReportID, Offset, and Type. Return NULL
  * if not found.
  * -------------------------------------------------------------------------- */
-HIDData *FindObject_with_ID(HIDDesc *pDesc, u_char ReportID, u_char Offset, u_char Type)
+HIDData_t *FindObject_with_ID(HIDDesc_t *pDesc, u_char ReportID, u_char Offset, u_char Type)
 {
-  HIDData *pData;
+  HIDData_t *pData;
   int i;
 
   for (i=0; i<pDesc->nitems; i++)
@@ -456,7 +444,7 @@ HIDData *FindObject_with_ID(HIDDesc *pDesc, u_char ReportID, u_char Offset, u_ch
  * Use Value, Offset, Size and LogMax of pData.
  * Return response in Value.
  * -------------------------------------------------------------------------- */
-void GetValue(const u_char* Buf, HIDData* pData, long *pValue)
+void GetValue(const u_char* Buf, HIDData_t* pData, long *pValue)
 {
   int Bit = pData->Offset + 8; /* First byte of report is report ID */
   int Weight=0;
@@ -555,7 +543,7 @@ void GetValue(const u_char* Buf, HIDData* pData, long *pValue)
  * Set a data in a report stored in Buf. Use Value, Offset and Size of pData.
  * Return response in Buf.
  * -------------------------------------------------------------------------- */
-void SetValue(const HIDData* pData, u_char* Buf, long Value)
+void SetValue(const HIDData_t* pData, u_char* Buf, long Value)
 {
   int Bit = pData->Offset + 8; /* First byte of report is report ID */
   int Weight=0;
@@ -583,42 +571,54 @@ void SetValue(const HIDData* pData, u_char* Buf, long Value)
    Output: parsed data structure. Returns allocated HIDDesc structure
    on success, NULL on failure with errno set. Note: the value
    returned by this function must be freed with Free_ReportDesc(). */
-HIDDesc *Parse_ReportDesc(u_char *ReportDesc, int n) {
-	HIDParser parser;
-	HIDData FoundData;
-	HIDData *item = NULL;
-	HIDData *r;
-	HIDDesc *pDesc;
+HIDDesc_t *Parse_ReportDesc(u_char *ReportDesc, int n) {
+	HIDParser_t *parser;
+	HIDData_t *item = NULL;
+	HIDDesc_t *pDesc;
 	int i, id, max;
 
-	pDesc = malloc(sizeof(HIDDesc));
+	pDesc = calloc(1, sizeof(*pDesc));
 	if (!pDesc) {
 		return NULL;
 	}
 
-	ResetParser(&parser);
-	memcpy(parser.ReportDesc, ReportDesc, n);
-	parser.ReportDescSize = n;
-
-	i=0;
-	while (HIDParse(&parser, &FoundData)) {
-		i++;
-		r = realloc(item, i*sizeof(HIDData));
-		if (!r) {
-			free(pDesc);
-			free(item);
-			return NULL;
-		}
-		item = r;
-		memcpy(&item[i-1], &FoundData, sizeof(HIDData));
+	pDesc->item = calloc(MAX_REPORT, sizeof(*pDesc->item));
+	if (!pDesc->item) {
+		Free_ReportDesc(pDesc);
+		return NULL;
 	}
+
+	parser = calloc(1, sizeof(*parser));
+	if (!parser) {
+		Free_ReportDesc(pDesc);
+		return NULL;
+	}
+
+	parser->ReportDesc = ReportDesc;
+	parser->ReportDescSize = n;
+
+	for (i = 0; i < MAX_REPORT; i++) {
+		if (!HIDParse(parser, &pDesc->item[i])) {
+			break;
+		}
+	}
+
+	free(parser);
+
+	if (i > 0) {
+		item = realloc(pDesc->item, i * sizeof(*pDesc->item));
+	}
+	if (!item) {
+		Free_ReportDesc(pDesc);
+		return NULL;
+	}
+
 	pDesc->nitems = i;
 	pDesc->item = item;
 
 	/* done scanning report descriptor; now calculate derived data */
 
 	/* make a list of reports and their lengths */
-	memset(pDesc->replen, 0, 256);
 	for (i=0; i<pDesc->nitems; i++) {
 		id = item[i].ReportID;
 
@@ -638,10 +638,10 @@ HIDDesc *Parse_ReportDesc(u_char *ReportDesc, int n) {
 }
 
 /* free a parsed report descriptor, as allocated by Parse_ReportDesc() */
-void Free_ReportDesc(HIDDesc *pDesc) {
-	if (pDesc) {
-		free(pDesc->item);
+void Free_ReportDesc(HIDDesc_t *pDesc) {
+	if (!pDesc) {
+		return;
 	}
+	free(pDesc->item);
 	free(pDesc);
 }
-
