@@ -1,9 +1,9 @@
 /*!
  * @file libshut.c
- * @brief HID Library - Serial SHUT backend for Generic HID Access (using MGE HIDParser)
+ * @brief HID Library - Serial SHUT communication sub driver
  *
- * @author Copyright (C) 2006
- *	Arnaud Quette <aquette.dev@gmail.com>
+ * @author Copyright (C)
+ *  2006 - 2007 Arnaud Quette <aquette.dev@gmail.com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -21,31 +21,26 @@
  *
  * -------------------------------------------------------------------------- */
 
-/* FIXME: remainder
- * - complete shut_set_report
- * - complete libshut_get_interrupt
+/* TODO list
+ * - cleanup, cleanup, cleanup
+ * - manage interrupt and complete libshut_get_interrupt / shut_control_msg routing
  * - complete shut_strerror
- * - complete commands and data table in mge-hid from mge-shut
+ * - validate / complete commands and data table in mge-hid from mge-shut
  */
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <sys/ioctl.h>
 #include <sys/types.h>
 #include <string.h>
 
-#ifdef HAVE_STDINT_H
-#include <stdint.h> /* for uint8_t, uint16_t */
-#endif
+#include "nut_stdint.h" /* for uint8_t, uint16_t, uint32_t */
 
 #include "serial.h"
 #include "libshut.h"
-#include "libhid.h"
-
 #include "common.h" /* for xmalloc, upsdebugx prototypes */
 
 #define SHUT_DRIVER_NAME	"MGE SHUT communication driver"
-#define SHUT_DRIVER_VERSION	"0.67"
+#define SHUT_DRIVER_VERSION	"0.80"
 #define MAX_TRY 4
 
 /*!
@@ -173,17 +168,18 @@ struct my_hid_descriptor {
 /*!
  * SHUT functions for HID marshalling
  */
-int shut_get_descriptor(shut_dev_handle *sdev, unsigned char type,
+int shut_get_descriptor(int upsfd, unsigned char type,
 			unsigned char index, void *buf, int size);
-int shut_get_string_simple(shut_dev_handle *dev, int index,
-			   unsigned char *buf, size_t buflen);
-int libshut_get_report(shut_dev_handle *devp, int ReportId,
+int shut_get_string_simple(int upsfd, int index,
+			   char *buf, size_t buflen);
+int libshut_get_report(int upsfd, int ReportId,
 		       unsigned char *raw_buf, int ReportSize );
-int shut_set_report(shut_dev_handle *sdev, int id, unsigned char *pkt, int reportlen);
-int libshut_get_interrupt(shut_dev_handle *devp, unsigned char *buf,
+int shut_set_report(int upsfd, int id, unsigned char *pkt, int reportlen);
+int libshut_get_interrupt(int upsfd, unsigned char *buf,
 			  int bufsize, int timeout);
-int shut_control_msg(shut_dev_handle *dev, int requesttype, int request,
+int shut_control_msg(int upsfd, int requesttype, int request,
 		     int value, int index, unsigned char *bytes, int size, int timeout);
+void libshut_close(int upsfd);
 
 /* FIXME */
 char * shut_strerror() { return ""; }
@@ -193,7 +189,7 @@ char * shut_strerror() { return ""; }
  * sync'ed with libusb
  */
 
-typedef struct shut_ctrltransfer {
+typedef struct shut_ctrltransfer_s {
 	uint8_t  bRequestType;
 	uint8_t  bRequest;
 	uint16_t wValue;
@@ -208,30 +204,30 @@ typedef struct shut_ctrltransfer {
 } shut_ctrltransfer_t;
 
 
-typedef union hid_data {
+typedef union hid_data_t {
 	shut_ctrltransfer_t  hid_pkt;
 	uint8_t raw_pkt[8]; /* max report lengh, was 8 */
-} hid_data_u;
+} hid_data_t;
 
-typedef struct shut_packet {
+typedef struct shut_packet_s {
 	uint8_t bType;
 	uint8_t bLength;
-	hid_data_u    data;
+	hid_data_t    data;
 	uint8_t bChecksum;
 } shut_packet_t;
 
-typedef union shut_data {
+typedef union shut_data_t {
 	shut_packet_t shut_pkt;
 	uint8_t raw_pkt[11];
-} shut_data_u;
+} shut_data_t;
 
-typedef union hid_desc_data {
+typedef union hid_desc_data_t {
 	struct my_hid_descriptor hid_desc;
 	uint8_t    raw_desc[9]; /* max report lengh, aws 9 */
-} hid_desc_data_u;
+} hid_desc_data_t;
 
 /* Device descriptor */
-typedef struct device_descriptor {
+typedef struct device_descriptor_s {
 	uint8_t  bLength;
 	uint8_t  bDescriptorType;
 	uint16_t bcdUSB;
@@ -248,25 +244,25 @@ typedef struct device_descriptor {
 	uint8_t  bNumConfigurations;
 } device_descriptor_t;
 #if 0
-typedef union device_desc_data {
+typedef union device_desc_data_t {
 	device_descriptor_t dev_desc;
 	uint8_t       raw_desc[18];
-} device_desc_data_u;
+} device_desc_data_t;
 #endif
 /* Low level SHUT (Serial HID UPS Transfer) routines  */
-void setline (int fd, int set);
-bool shut_synchronise (int fd);
-int shut_wait_ack (int fd);
-int shut_interrupt_read(shut_dev_handle *dev, int ep, unsigned char *bytes,
+void setline(int upsfd, int set);
+int shut_synchronise(int upsfd);
+int shut_wait_ack(int upsfd);
+int shut_interrupt_read(int upsfd, int ep, unsigned char *bytes,
 			int size, int timeout);
-int shut_control_msg(shut_dev_handle *dev, int requesttype, int request, int value,
+int shut_control_msg(int upsfd, int requesttype, int request, int value,
 		     int index, unsigned char *bytes, int size, int timeout);
 
 
 /* Data portability */
 /* realign packet data according to Endianess */
 #define BYTESWAP(in) (((in & 0xFF) << 8) + ((in & 0xFF00) >> 8))
-static void align_request(struct shut_ctrltransfer *ctrl )
+static void align_request(struct shut_ctrltransfer_s *ctrl )
 {
 #if WORDS_BIGENDIAN
         /* Sparc/Mips/... are big endian, USB/SHUT little endian */
@@ -276,44 +272,56 @@ static void align_request(struct shut_ctrltransfer *ctrl )
 #endif
 }
 
-/* return report descriptor on success, NULL otherwise */
-/* mode: MODE_OPEN for the 1rst time, MODE_REOPEN to skip getting
-    report descriptor (the longer part). On success, fill in the
-    curDevice structure and return the report descriptor length. On
-    failure, return -1. Note: ReportDesc must point to a large enough
-    buffer. There's no way to know the size ahead of time. Matcher is
-    a linked list of matchers (see libhid.h), and the opened device
-    must match all of them. */
-int libshut_open(shut_dev_handle **sdevp, HIDDevice *curDevice,
-		 HIDDeviceMatcher_t *matcher, unsigned char *ReportDesc, int mode)
+/* On success, fill in the curDevice structure and return the report
+ * descriptor length. On failure, return -1.
+ * Note: When callback is not NULL, the report descriptor will be
+ * passed to this function together with the upsfd and SHUTDevice_t
+ * information. This callback should return a value > 0 if the device
+ * is accepted, or < 1 if not.
+ */
+int libshut_open(int *upsfd, SHUTDevice_t *curDevice, char *device_path,
+	int (*callback)(int upsfd, SHUTDevice_t *hd, unsigned char *rdbuf, int rdlen))
 {
 	int ret, res; 
 	unsigned char buf[20];
 	char string[256];
 	struct my_hid_descriptor *desc;
-	struct device_descriptor *dev_descriptor;
-	shut_dev_handle *devp = *sdevp;
-	
-	upsdebugx(2, "libshut_open: using port %s", devp->device_path);
+	struct device_descriptor_s *dev_descriptor;
 	
+	/* report descriptor */
+	unsigned char	rdbuf[MAX_REPORT_SIZE];
+	int		rdlen;
+
+	upsdebugx(2, "libshut_open: using port %s", device_path);
+
+	/* If device is still open, close it */
+	if (*upsfd > 0) {
+		ser_close(*upsfd, device_path);
+	}
+
 	/* initialize serial port */
 	/* FIXME: add variable baudrate detection */
-	devp->upsfd = ser_open(devp->device_path);
-	ser_set_speed(devp->upsfd, devp->device_path, B2400);
-	setline (devp->upsfd, 1);
+	*upsfd = ser_open(device_path);
+	ser_set_speed(*upsfd, device_path, B2400);
+	setline(*upsfd, 1);
 
 	/* initialise communication */
-	if (!shut_synchronise (devp->upsfd))
+	if (!shut_synchronise(*upsfd))
 	{
 		upsdebugx(2, "No communication with UPS");
 		return -1;
 	}
-	else
-		upsdebugx(2, "Communication with UPS established");
+
+	upsdebugx(2, "Communication with UPS established");
+
+	/* we can skip the rest due to serial bus specifics! */
+	if (!callback) {
+		return 1;
+	}
 
 	/* Get DEVICE descriptor */
-	dev_descriptor = (struct device_descriptor *)buf;
-	res = shut_get_descriptor(devp, USB_DT_DEVICE, 0, buf, USB_DT_DEVICE_SIZE);
+	dev_descriptor = (struct device_descriptor_s *)buf;
+	res = shut_get_descriptor(*upsfd, USB_DT_DEVICE, 0, buf, USB_DT_DEVICE_SIZE);
 	/* res = shut_control_msg(devp, USB_ENDPOINT_IN+1, USB_REQ_GET_DESCRIPTOR,
 	(USB_DT_DEVICE << 8) + 0, 0, buf, 0x9, SHUT_TIMEOUT); */
 	if (res < 0)
@@ -321,12 +329,13 @@ int libshut_open(shut_dev_handle **sdevp, HIDDevice *curDevice,
 		upsdebugx(2, "Unable to get DEVICE descriptor (%s)", shut_strerror());
 		return -1;
 	}
-	else if (res < 9)
+
+	if (res < 9)
 	{
 		upsdebugx(2, "DEVICE descriptor too short (expected %d, got %d)",
 		      USB_DT_DEVICE_SIZE, res);
 		return -1;
-	} 
+	}
 
 	/* collect the identifying information of this
 		device. Note that this is safe, because
@@ -334,43 +343,52 @@ int libshut_open(shut_dev_handle **sdevp, HIDDevice *curDevice,
 		this (and therefore we do not yet need to
 		detach any kernel drivers). */
 
+	free(curDevice->Vendor);
+	free(curDevice->Product);
+	free(curDevice->Serial);
+	free(curDevice->Bus);
+	memset(curDevice, '\0', sizeof(*curDevice));
+
 	curDevice->VendorID = dev_descriptor->idVendor;
 	curDevice->ProductID = dev_descriptor->idProduct;
 	curDevice->Vendor = strdup("MGE UPS SYSTEMS");
-	curDevice->Product = strdup("unknown");
-	curDevice->Serial = strdup("unknown");
-	curDevice->Bus = strdup("shut");
+	curDevice->Bus = strdup("serial");
 
-	if (dev_descriptor->iProduct)
-	{
-		ret = shut_get_string_simple(devp, dev_descriptor->iProduct,
-					     string, 0x25);
-		if (ret > 0)
-			curDevice->Product = strdup(string);
+	if (dev_descriptor->iProduct) {
+		ret = shut_get_string_simple(*upsfd, dev_descriptor->iProduct, string, 0x25);
+	} else {
+		ret = 0;
 	}
 
-	if (dev_descriptor->iSerialNumber)
-	{
-		ret = shut_get_string_simple(devp, dev_descriptor->iSerialNumber,
-					     string, 0x25);
-		if (ret > 0)
-			curDevice->Serial = strdup(string);
+	if (ret > 0) {
+		curDevice->Product = strdup(string);
+	} else {
+		curDevice->Product = strdup("unknown");
+	}
+
+	if (dev_descriptor->iSerialNumber) {
+		ret = shut_get_string_simple(*upsfd, dev_descriptor->iSerialNumber, string, 0x25);
+	} else {
+		ret = 0;
+	}
+
+	if (ret > 0) {
+		curDevice->Serial = strdup(string);
+	} else {
+		curDevice->Serial = strdup("unknown");
 	}
 
 	upsdebugx(2, "- VendorID: %04x", curDevice->VendorID);
 	upsdebugx(2, "- ProductID: %04x", curDevice->ProductID);
-	upsdebugx(2, "- Manufacturer: %s", curDevice->Vendor ? curDevice->Vendor : "unknown");
-	upsdebugx(2, "- Product: %s", curDevice->Product ? curDevice->Product : "unknown");
-	upsdebugx(2, "- Serial Number: %s", curDevice->Serial ? curDevice->Serial : "unknown");
-	upsdebugx(2, "- Bus: %s", curDevice->Bus ? curDevice->Bus : "unknown");
+	upsdebugx(2, "- Manufacturer: %s", curDevice->Vendor);
+	upsdebugx(2, "- Product: %s", curDevice->Product);
+	upsdebugx(2, "- Serial Number: %s", curDevice->Serial);
+	upsdebugx(2, "- Bus: %s", curDevice->Bus);
 	upsdebugx(2, "Device matches");
 
-	if (mode == MODE_REOPEN)
-		return 1;
-
 	/* Get HID descriptor */
 	desc = (struct my_hid_descriptor *)buf;
-	res = shut_get_descriptor(devp, USB_DT_HID, 0, buf, 0x9);
+	res = shut_get_descriptor(*upsfd, USB_DT_HID, 0, buf, 0x9);
 	/* res = shut_control_msg(devp, USB_ENDPOINT_IN+1, USB_REQ_GET_DESCRIPTOR,
 			(USB_DT_HID << 8) + 0, 0, buf, 0x9, SHUT_TIMEOUT); */
 
@@ -379,7 +397,8 @@ int libshut_open(shut_dev_handle **sdevp, HIDDevice *curDevice,
 		upsdebugx(2, "Unable to get HID descriptor (%s)", shut_strerror());
 		return -1;
 	}
-	else if (res < 9)
+
+	if (res < 9)
 	{
 		upsdebugx(2, "HID descriptor too short (expected %d, got %d)", 8, res);
 		return -1;
@@ -394,29 +413,40 @@ int libshut_open(shut_dev_handle **sdevp, HIDDevice *curDevice,
 		return -1;
 	}*/
 
+	rdlen = desc->wDescriptorLength;
+
+	if (rdlen > (int)sizeof(rdbuf)) {
+		upsdebugx(2, "HID descriptor too long %d (max %d)", rdlen, sizeof(rdbuf));
+		return -1;
+	}
+
 	/* Get REPORT descriptor */
-	res = shut_get_descriptor(devp, USB_DT_REPORT, 0, ReportDesc,
-				   desc->wDescriptorLength);
+	res = shut_get_descriptor(*upsfd, USB_DT_REPORT, 0, rdbuf, rdlen);
 	/* res = shut_control_msg(devp, USB_ENDPOINT_IN+1, USB_REQ_GET_DESCRIPTOR,
 				(USB_DT_REPORT << 8) + 0, 0, ReportDesc, 
 			desc->wDescriptorLength, SHUT_TIMEOUT); */
-	if (res >= desc->wDescriptorLength) 
+	if (res == rdlen)
 	{
-		upsdebugx(2, "Report descriptor retrieved (Reportlen = %u)",
-		      desc->wDescriptorLength);
+		res = callback(*upsfd, curDevice, rdbuf, rdlen);
+		if (res < 1) {
+			upsdebugx(2, "Caller doesn't like this device");
+			return -1;
+		}
+
+		upsdebugx(2, "Report descriptor retrieved (Reportlen = %d)", rdlen);
 		upsdebugx(2, "Found HID device");
 		fflush(stdout);
 
-		return desc->wDescriptorLength;
+		return rdlen;
 	}
-	else if (res < 0)
+
+	if (res < 0)
 	{
 		upsdebugx(2, "Unable to get Report descriptor (%d)", res);
 	}
 	else
 	{
-		upsdebugx(2, "Report descriptor too short (expected %d, got %d)",
-		      desc->wDescriptorLength, res);
+		upsdebugx(2, "Report descriptor too short (expected %d, got %d)", rdlen, res);
 	}
 	
 	upsdebugx(2, "No appropriate HID device found");
@@ -425,88 +455,97 @@ int libshut_open(shut_dev_handle **sdevp, HIDDevice *curDevice,
 	return -1;
 }
 
-void libshut_close(shut_dev_handle *sdev)
+void libshut_close(int upsfd)
 {
-	ser_close(sdev->upsfd, sdev->device_path);
+	if (upsfd < 1) {
+		return;
+	}
+
+	ser_close(upsfd, NULL);
 }
 
 /* return the report of ID=type in report 
  * return -1 on failure, report length on success
  */
-int libshut_get_report(shut_dev_handle *devp, int ReportId,
+int libshut_get_report(int upsfd, int ReportId,
 		       unsigned char *raw_buf, int ReportSize )
 {
+	if (upsfd < 1) {
+		return 0;
+	}
+
 	upsdebugx(4, "Entering libshut_get_report");
 
-	if (devp != NULL)
-	{
-		return shut_control_msg(devp,
-			REQUEST_TYPE_GET_REPORT,
-			// == USB_ENDPOINT_IN + USB_TYPE_CLASS + USB_RECIP_INTERFACE,
-			 0x01,
-			 ReportId+(0x03<<8), /* HID_REPORT_TYPE_FEATURE */
-			 0, raw_buf, ReportSize, SHUT_TIMEOUT);
-	}
-	else
-		return 0;
+	return shut_control_msg(upsfd,
+		REQUEST_TYPE_GET_REPORT,
+		/* == USB_ENDPOINT_IN + USB_TYPE_CLASS + USB_RECIP_INTERFACE, */
+		 0x01,
+		 ReportId+(0x03<<8), /* HID_REPORT_TYPE_FEATURE */
+		 0, raw_buf, ReportSize, SHUT_TIMEOUT);
 }
 
-int libshut_set_report(shut_dev_handle *devp, int ReportId,
+/* return ReportSize upon success ; -1 otherwise */
+int libshut_set_report(int upsfd, int ReportId,
 		       unsigned char *raw_buf, int ReportSize )
 {
-	int ret = 0;
+	int ret;
 	
-	upsdebugx(1, "Entering libshut_set_report");
-
-	if (devp != NULL)
-	{
-		ret = shut_control_msg(devp, 
-			REQUEST_TYPE_SET_REPORT,
-			// == USB_ENDPOINT_OUT + USB_TYPE_CLASS + USB_RECIP_INTERFACE,
-			0x09,
-			ReportId+(0x03<<8), /* HID_REPORT_TYPE_FEATURE */
-			0, raw_buf, ReportSize, SHUT_TIMEOUT);
+	if (upsfd < 1) {
+		return 0;
 	}
-exit(0);
-	return ret;
+		
+	upsdebugx(1, "Entering libshut_set_report (report %x, len %i)",
+		ReportId, ReportSize);
+
+	upsdebug_hex (4, "==> Report after set", raw_buf, ReportSize);
+
+	ret = shut_control_msg(upsfd, 
+		REQUEST_TYPE_SET_REPORT,
+		/* == USB_ENDPOINT_OUT + USB_TYPE_CLASS + USB_RECIP_INTERFACE, */
+		0x09,
+		ReportId+(0x03<<8), /* HID_REPORT_TYPE_FEATURE */
+		0, raw_buf, ReportSize, SHUT_TIMEOUT);
+	
+	return ((ret == 0) ? ReportSize : ret);
 }
 
-int libshut_get_string(shut_dev_handle *devp, int StringIdx, char *buf)
+int libshut_get_string(int upsfd, int StringIdx, char *buf, size_t buflen)
 {
-	int ret = -1;
+	int ret;
 	
-	if (devp != NULL)
-	{
-		ret = shut_get_string_simple(devp, StringIdx, buf, 20);
-		 /* sizeof(buf)); */
-		if (ret > 0)
-			upsdebugx(2, "-> String: %s (len = %i/%i)",
-			      buf, ret, sizeof(buf));
-		else
-			upsdebugx(2, "- Unable to fetch buf");
+	if (upsfd < 1) {
+		return -1;
 	}
+
+	ret = shut_get_string_simple(upsfd, StringIdx, buf, buflen);
+	if (ret > 0)
+		upsdebugx(2, "-> String: %s (len = %i/%i)", buf, ret, buflen);
+	else
+		upsdebugx(2, "- Unable to fetch buf");
+
 	return ret;
 }
 
-int libshut_get_interrupt(shut_dev_handle *devp, unsigned char *buf,
+int libshut_get_interrupt(int upsfd, unsigned char *buf,
 			   int bufsize, int timeout)
 {
-  int ret = -1;
+	int ret;
 
-  if (devp != NULL)
-	{
-	  /* FIXME: hardcoded interrupt EP => need to get EP descr for IF descr */
-	  ret = shut_interrupt_read(devp, 0x81, buf, bufsize, timeout);
-	  if (ret > 0)
-		upsdebugx(6, " ok");
-	  else
-		upsdebugx(6, " none (%i)", ret);
+	if (upsfd < 1) {
+		return -1;
 	}
 
-  return ret;
+	/* FIXME: hardcoded interrupt EP => need to get EP descr for IF descr */
+	ret = shut_interrupt_read(upsfd, 0x81, buf, bufsize, timeout);
+	if (ret > 0)
+		upsdebugx(6, " ok");
+	else
+		upsdebugx(6, " none (%i)", ret);
+	
+	return ret;
 }
 
-communication_subdriver_t shut_subdriver = {
+shut_communication_subdriver_t shut_subdriver = {
 	SHUT_DRIVER_NAME,
 	SHUT_DRIVER_VERSION,
 	libshut_open,
@@ -527,21 +566,20 @@ communication_subdriver_t shut_subdriver = {
  * set : 1 to set comm
  * set : 0 to stop commupsh.
  */
-void setline (int fd, int set)
+void setline(int upsfd, int set)
 {
-	int dtr = TIOCM_DTR;
-	int rts = TIOCM_RTS;
-	
+	if (upsfd < 1) {
+		return;
+	}
+
 	upsdebugx(3, "entering setline(%i)", set);
 
 	if (set == 1) {
-		ioctl (fd, TIOCMBIC, &dtr);
-		ioctl (fd, TIOCMBIS, &rts);
-	}
-	else
-	{
-		ioctl (fd, TIOCMBIS, &dtr);
-		ioctl (fd, TIOCMBIC, &rts);
+		ser_set_dtr(upsfd, 0);
+		ser_set_rts(upsfd, 1);
+	} else {
+		ser_set_dtr(upsfd, 1);
+		ser_set_rts(upsfd, 0);
 	}
 }
 
@@ -553,9 +591,9 @@ void setline (int fd, int set)
  * return TRUE on success, FALSE on failure
  *
  *****************************************************************************/
-bool shut_synchronise (int fd)
+int shut_synchronise(int upsfd)
 {
-	bool retCode = FALSE;
+	int retCode = 0;
 	u_char c = SHUT_SYNC, reply;
 	int try;
 		
@@ -583,19 +621,17 @@ bool shut_synchronise (int fd)
 	{
 		upsdebugx (3, "Syncing communication (try %i)", try);
 
-		if ((ser_send_char(fd, c)) == -1)
+		if ((ser_send_char(upsfd, c)) == -1)
 		{
 			upsdebugx (3, "Communication error while writing to port");
-			retCode = FALSE;
+			continue;
 		}
-		else
+
+		ser_get_char(upsfd, &reply, 1, 0);
+		if (reply == c)
 		{
-			ser_get_char(fd, &reply, 1, 0);
-			if (reply == c)
-			{
-				upsdebugx (3, "Syncing and notification setting done");
-				retCode = TRUE;
-			}
+			upsdebugx (3, "Syncing and notification setting done");
+			return 1;
 		}
 	}
 	return retCode;
@@ -617,7 +653,7 @@ u_char shut_checksum(const u_char *buf, int bufsize)
 }
 
 
-int shut_packet_recv (int fd, u_char *Buf, int datalen)
+int shut_packet_recv(int upsfd, u_char *Buf, int datalen)
 {
 	u_char   Start[2];
 	u_char   Frame[8];
@@ -626,14 +662,14 @@ int shut_packet_recv (int fd, u_char *Buf, int datalen)
 	u_short  Pos=0;
 	u_char   Retry=0;
 	int recv;
-	shut_data_u   sdata;
+	shut_data_t   sdata;
 	
 	upsdebugx (4, "entering shut_packet_recv (%i)", datalen);
 	
 	while(datalen>0 && Retry<3)
 	{
-		/// if(serial_read (SHUT_TIMEOUT, &Start[0]) >= 0)
-		if(ser_get_char(fd, &Start[0], SHUT_TIMEOUT/1000, 0) >= 0)
+		/* if(serial_read (SHUT_TIMEOUT, &Start[0]) >= 0) */
+		if(ser_get_char(upsfd, &Start[0], SHUT_TIMEOUT/1000, 0) >= 0)
 		{
 			sdata.shut_pkt.bType = Start[0];
 			if(Start[0]==SHUT_SYNC)
@@ -644,8 +680,8 @@ int shut_packet_recv (int fd, u_char *Buf, int datalen)
 			}
 			else 
 			{
-				/// if((serial_read (SHUT_TIMEOUT, &Start[1]) >= 0) &&
-				if( (ser_get_char(fd, &Start[1], SHUT_TIMEOUT/1000, 0) >= 0) &&
+				/* if((serial_read (SHUT_TIMEOUT, &Start[1]) >= 0) && */
+				if( (ser_get_char(upsfd, &Start[1], SHUT_TIMEOUT/1000, 0) >= 0) &&
 							((Start[1]>>4)==(Start[1]&0x0F)))
 				{
 					upsdebug_hex(4, "Receive", Start, 2);
@@ -653,14 +689,14 @@ int shut_packet_recv (int fd, u_char *Buf, int datalen)
 					sdata.shut_pkt.bLength = Size;
 					for(recv=0;recv<Size;recv++)
 					{
-						///if(serial_read (SHUT_TIMEOUT, &Frame[recv]) < 0)
-						if(ser_get_char(fd, &Frame[recv], SHUT_TIMEOUT/1000, 0) < 0)
+						/* if(serial_read (SHUT_TIMEOUT, &Frame[recv]) < 0) */
+						if(ser_get_char(upsfd, &Frame[recv], SHUT_TIMEOUT/1000, 0) < 0)
 							break;
 					}
 					upsdebug_hex(4, "Receive", Frame, Size);
 
-					///serial_read (SHUT_TIMEOUT, &Chk[0]);
-					ser_get_char(fd, &Chk[0], SHUT_TIMEOUT/1000, 0);
+					/* serial_read (SHUT_TIMEOUT, &Chk[0]); */
+					ser_get_char(upsfd, &Chk[0], SHUT_TIMEOUT/1000, 0);
 					if(Chk[0]==shut_checksum(Frame, Size))
 					{
 						upsdebugx (4, "shut_checksum: %02x => OK", Chk[0]);
@@ -670,8 +706,8 @@ int shut_packet_recv (int fd, u_char *Buf, int datalen)
 						Pos+=Size;
 						Retry=0;
 					
-						ser_send_char(fd, SHUT_OK);
-						//shut_token_send(SHUT_OK);
+						ser_send_char(upsfd, SHUT_OK);
+						/* shut_token_send(SHUT_OK); */
 
 						if(Start[0]&SHUT_PKT_LAST)
 						{
@@ -691,8 +727,8 @@ int shut_packet_recv (int fd, u_char *Buf, int datalen)
 					else
 					{
 						upsdebugx (4, "shut_checksum: %02x => NOK", Chk[0]);
-						ser_send_char(fd, SHUT_NOK);
-						//shut_token_send(SHUT_NOK);
+						ser_send_char(upsfd, SHUT_NOK);
+						/* shut_token_send(SHUT_NOK); */
 						Retry++;
 					}
 				}
@@ -708,21 +744,24 @@ int shut_packet_recv (int fd, u_char *Buf, int datalen)
 }
 
 /**********************************************************************/
-int shut_interrupt_read(shut_dev_handle *dev, int ep, unsigned char *bytes, int size,
+int shut_interrupt_read(int upsfd, int ep, unsigned char *bytes, int size,
 		       int timeout)
 {
+/*
+	usleep(timeout * 1000);
+ */
 	/* FIXME: to be written */
 	return 0;
 }
 
 /**********************************************************************/
-int shut_get_string_simple(shut_dev_handle *dev, int index,
-			   unsigned char *buf, size_t buflen)
+int shut_get_string_simple(int upsfd, int index,
+			   char *buf, size_t buflen)
 {
-	char tbuf[255];       /* Some devices choke on size > 255 */
+	unsigned char tbuf[255];       /* Some devices choke on size > 255 */
 	int ret, si, di;
 	
-	ret = shut_control_msg(dev, USB_ENDPOINT_IN, USB_REQ_GET_DESCRIPTOR,
+	ret = shut_control_msg(upsfd, USB_ENDPOINT_IN, USB_REQ_GET_DESCRIPTOR,
 			(USB_DT_STRING << 8) + index, 0x0, tbuf, buflen, 1000);
 	if (ret < 0)
 		return ret;
@@ -736,7 +775,7 @@ int shut_get_string_simple(shut_dev_handle *dev, int index,
 	/* skip the zero'ed high bytes */
 	for (di = 0, si = 2; si < tbuf[0]; si += 2)
 	{
-		if (di >= (buflen - 1))
+		if (di >= (int)(buflen - 1))
 			break;
 
 		if (tbuf[si + 1])   /* high byte */
@@ -764,30 +803,38 @@ int shut_get_string_simple(shut_dev_handle *dev, int index,
  * return 0 on success, -1 on failure, -2 on NACK
  *
  *********************************************************************/
-int shut_get_descriptor(shut_dev_handle *sdev, unsigned char type,
+int shut_get_descriptor(int upsfd, unsigned char type,
 			unsigned char index, void *buf, int size)
 {
 	memset(buf, 0, size);
 
 	upsdebugx (2, "entering shut_get_descriptor(n %02x, %i)", type, size);
 
-	return shut_control_msg(sdev, USB_ENDPOINT_IN+(type>=USB_DT_HID?1:0),
+	return shut_control_msg(upsfd, USB_ENDPOINT_IN+(type>=USB_DT_HID?1:0),
 				 USB_REQ_GET_DESCRIPTOR, (type << 8) + index, 0, buf, size, 1000);
 }
 
 /* Take care of a SHUT transfer (sending and receiving data) */
-int shut_control_msg(shut_dev_handle *sdev, int requesttype, int request,
+int shut_control_msg(int upsfd, int requesttype, int request,
 		    int value, int index, unsigned char *bytes, int size, int timeout)
 {
 	unsigned char shut_pkt[11];
-	short Retry=1;
-	short data_size, remaining_size = size; //Size;
+	short Retry=1, set_pass = -1;
+	short data_size, remaining_size = size;
 	int i;
-	struct shut_ctrltransfer ctrl;
-	int ret;
+	struct shut_ctrltransfer_s ctrl;
+	int ret = 0;
 	
 	upsdebugx (3, "entering shut_control_msg");
-		
+
+	/* deal for set requests */
+	if (requesttype == REQUEST_TYPE_SET_REPORT)
+	{
+		set_pass = 1;
+		/* add 8 for the first frame that declares a coming set */
+		remaining_size+= 8;
+	}
+
 	/* build the control request */
 	ctrl.bRequestType = requesttype;
 	ctrl.bRequest = request;
@@ -800,16 +847,26 @@ int shut_control_msg(shut_dev_handle *sdev, int requesttype, int request,
 	align_request(&ctrl);
 
 	/* Send all data */
-	while(remaining_size > 0 && Retry > 0)
-        {
-		data_size = (size >= 8) ? 8 : remaining_size;
+	while(remaining_size > 0 && Retry > 0) {
+
+		if (requesttype == REQUEST_TYPE_SET_REPORT) {
+			if (set_pass == 1) {
+				data_size = 8;
+				set_pass++; /* prepare for the next step */
+			}
+			else {
+				data_size = size;
+				upsdebug_hex(4, "data", bytes, data_size);
+			}
+		}
+		else
+			data_size = (size >= 8) ? 8 : remaining_size;
 		
 		/* Forge the SHUT Frame */
-		/* shut_pkt[0] = SHUT_TYPE_REQUEST + SHUT_PKT_LAST; */ //( (remaining_size>8)? 0 :
 		shut_pkt[0] = SHUT_TYPE_REQUEST + ( ((requesttype == REQUEST_TYPE_SET_REPORT) && (remaining_size>8))? 0 : SHUT_PKT_LAST);
 		shut_pkt[1] = (data_size<<4) + data_size;
 		if ( (requesttype == REQUEST_TYPE_SET_REPORT) && (remaining_size < 8) )
-			memcpy(&shut_pkt[2], &bytes, remaining_size); // we need to send ctrl.data 
+			memcpy(&shut_pkt[2], bytes, data_size); /* we need to send ctrl.data  */
 		else
 			memcpy(&shut_pkt[2], &ctrl, 8);
 		shut_pkt[(data_size+3) - 1] = shut_checksum(&shut_pkt[2], data_size);
@@ -818,16 +875,20 @@ int shut_control_msg(shut_dev_handle *sdev, int requesttype, int request,
 		* NACK handling should take care of the rest */
 		if (Retry == 1)
 		{
-			ser_send_buf(sdev->upsfd, shut_pkt, data_size+3);
+			ser_send_buf(upsfd, shut_pkt, data_size+3);
 			upsdebug_hex(3, "shut_control_msg", shut_pkt, data_size+3);
-			//serial_send (shut_pkt, data_size+3);
+			/* serial_send (shut_pkt, data_size+3); */
 		}
 
-		i = shut_wait_ack (sdev->upsfd);
+		i = shut_wait_ack (upsfd);
 		switch (i)
 		{
 			case 0:
-				remaining_size = 0; //remaining_size-=data_size;
+				if (requesttype == REQUEST_TYPE_SET_REPORT)
+					remaining_size-=data_size;
+				else
+					remaining_size = 0;
+
 				Retry=1;
 				break;
 			case -1:
@@ -837,14 +898,14 @@ int shut_control_msg(shut_dev_handle *sdev, int requesttype, int request,
 					
 					/* try to resync, and give one more try */
 					Retry--;
-					shut_synchronise(sdev->upsfd);
+					shut_synchronise(upsfd);
 					return i;
 				}
 				else
 				{
 					upsdebugx(4, "Retry = %i", Retry);
 					/* Send a NACK to get a resend from the UPS */
-					ser_send_char(sdev->upsfd, SHUT_NOK);
+					ser_send_char(upsfd, SHUT_NOK);
 					Retry++;
 				}
 				break;
@@ -857,47 +918,9 @@ int shut_control_msg(shut_dev_handle *sdev, int requesttype, int request,
 	if (remaining_size != 0)
 		return -1;
 	
-	
-	
 	/* now receive data, except for SET_REPORT */
 	if (requesttype != REQUEST_TYPE_SET_REPORT)
-		ret = shut_packet_recv (sdev->upsfd, bytes, size);
-	
-	
-	
-	
-	
-	
-	
-//	ret = shut_transfer(sdev->upsfd, &ctrl);
-	//if (ret < 0)
-	//	USB_ERROR_STR(-errno, "error sending control message: %s",
-	//		       strerror(errno));
-#if 0
-	// just shut_set_report : SHUT_PKT_LAST => 0 + 8 => sizeof(data)
-	if((ret = shut_packet_send (sdev, &ctrl, 8, SHUT_PKT_LAST)) > 0)
-	{
-		if((ret = shut_packet_recv (sdev, ctrl, reportlen)) > 0)
-		{
-			//upsdebug_hex(3, "shut_get_report", pkt, retcode)
-			;
-		}
-	}
-#endif
-	/* first packet to instruct a set command */
-// 	if((retcode = shut_packet_send (&data, sizeof(data), 0x0)) > 0)
-// 	{
-// 		/* second packet to give the actual data */
-// 		memcpy(&data.raw_pkt, pkt, reportlen);
-// 		upsdebug_hex(3, "Set2", pkt, reportlen);
-// 
-// 		retcode = shut_packet_send (&data, reportlen, SHUT_PKT_LAST);
-// 	}
-
-	// ret = ioctl(dev->fd, IOCTL_USB_CONTROL, &ctrl);
-	//if (ret < 0)
-	//	USB_ERROR_STR(-errno, "error sending control message: %s",
-	//		       strerror(errno));
+		ret = shut_packet_recv (upsfd, bytes, size);
 
 	return ret;
 }
@@ -910,12 +933,12 @@ int shut_control_msg(shut_dev_handle *sdev, int requesttype, int request,
  * returns 0 on success, -1 on error, -2 on NACK, -3 on NOTIFICATION
  * 
  *********************************************************************/
-int shut_wait_ack (int fd)
+int shut_wait_ack(int upsfd)
 {
 	int retCode = -1;
 	u_char c = '\0';
 
-	ser_get_char(fd, &c, SHUT_TIMEOUT/1000, 0);
+	ser_get_char(upsfd, &c, SHUT_TIMEOUT/1000, 0);
 	if (c == SHUT_OK)
 	{
 		upsdebugx (2, "shut_wait_ack(): ACK received");
