@@ -22,6 +22,12 @@
  */
 
 
+/*
+ * A document describing the protocol implemented by this driver can be
+ * found online at "http://www.networkupstools.org/protocols/megatec.html".
+ */
+
+
 #include "main.h"
 #include "serial.h"
 #include "megatec.h"
@@ -29,29 +35,30 @@
 #include <stdio.h>
 #include <limits.h>
 #include <string.h>
+#include <stdlib.h>
 
 
 #define ENDCHAR  '\r'
-#define IGNCHARS "(#"
+#define IGNCHARS ""
 
 #define RECV_BUFFER_LEN 128
 
-/* The expected reply lengths (without IGNCHARS) */
-#define F_CMD_REPLY_LEN  20
-#define Q1_CMD_REPLY_LEN 45
-#define I_CMD_REPLY_LEN 37
+/* The expected reply lengths */
+#define F_CMD_REPLY_LEN  21
+#define Q1_CMD_REPLY_LEN 46
+#define I_CMD_REPLY_LEN  38
 
 #define IDENT_MAXTRIES   5
 #define IDENT_MINSUCCESS 3
 
-#define SEND_PACE    100000  /* 100ms interval between chars */
-#define READ_TIMEOUT 2       /* 2 seconds timeout on read */
+#define SEND_PACE    100000  /* interval between chars on send (usec) */
+#define READ_TIMEOUT 2       /* timeout on read (seconds) */
 
 #define MAX_START_DELAY    9999
 #define MAX_SHUTDOWN_DELAY 99
 
 /* Maximum length of a string representing these values */
-#define MAX_START_DELAY_LEN 4
+#define MAX_START_DELAY_LEN    4
 #define MAX_SHUTDOWN_DELAY_LEN 2
 
 #define N_FLAGS 8
@@ -77,7 +84,7 @@ typedef struct {
 	char mfr[UPS_MFR_CHARS + 1];
 	char model[UPS_MODEL_CHARS + 1];
 	char version[UPS_VERSION_CHARS + 1];
-} UPSInfo;
+} UPSInfo_t;
 
 
 /* The values returned by the UPS for an "F" query */
@@ -86,7 +93,7 @@ typedef struct {
 	float current;
 	float battvolt;
 	float freq;
-} FirmwareValues;
+} FirmwareValues_t;
 
 
 /* The values returned by the UPS for an "Q1" query */
@@ -99,26 +106,31 @@ typedef struct {
 	float battvolt;
 	float temp;
 	char flags[N_FLAGS + 1];
-} QueryValues;
+} QueryValues_t;
 
 
 /* Parameters for known battery types */
 typedef struct {
-	int nominal;  /* battery voltage (nominal) */
-	float min;    /* lower bound for a single battery of "nominal" voltage (see "set_battery_params" below) */
-	float max;    /* upper bound for a single battery of "nominal" voltage (see "set_battery_params" below) */
-	float empty;  /* fully discharged battery */
-	float full;   /* fully charged battery */
-} BatteryVolts;
+	float nominal;  /* battery voltage (nominal) */
+	float min;      /* lower bound for a single battery of "nominal" voltage (see "set_battery_params" below) */
+	float max;      /* upper bound for a single battery of "nominal" voltage (see "set_battery_params" below) */
+	float empty;    /* fully discharged battery */
+	float full;     /* fully charged battery */
+	float low;      /* low battery (unused) */
+} BatteryVolts_t;
 
 
 /* Known battery types must be in ascending order by "nominal" first, and then by "max". */
-static BatteryVolts batteries[] = {{ 12,  9.0, 16.0,  9.7, 13.7 },   /* Mustek PowerMust 600VA Plus */
-                                   { 12, 18.0, 30.0, 18.8, 26.8 },   /* PowerWalker Line-Interactive VI 1000 (LB at 22.3V) */
-                                   { 24, 18.0, 30.0, 19.4, 27.4 },   /* Mustek PowerMust 1000VA Plus (LB at 22.2V) */
-                                   { 36,  1.5,  3.0, 1.64, 2.31 },   /* Mustek PowerMust 1000VA On-Line (LB at 1.88V) */
-                                   { 96,  1.5,  3.0, 1.63, 2.29 },   /* Ablerex MS3000RT (LB at 1.8V, 25% charge) */
-                                   {  0,  0.0,  0.0,  0.0,  0.0 }};  /* END OF DATA */
+static BatteryVolts_t batteries[] = {{ 12.0,  9.0, 16.0,  9.7, 13.7,  0.0 },   /* Mustek PowerMust 600VA Plus (LB unknown) */
+                                     { 12.0, 18.0, 30.0, 18.8, 26.8, 22.3 },   /* PowerWalker Line-Interactive VI 1000 */
+                                     { 23.5, 18.0, 30.0, 21.3, 27.1, 22.2 },   /* UNITEK ALPHA2600 */
+                                     { 24.0, 18.0, 30.0, 19.4, 27.4, 22.2 },   /* Mustek PowerMust 1000VA Plus */
+                                     { 36.0,  1.5,  3.0, 1.64, 2.31, 1.88 },   /* Mustek PowerMust 1000VA On-Line */
+                                     { 36.0, 30.0, 42.0, 32.5, 41.0,  0.0 },   /* Mecer ME-2000 (LB unknown) */
+                                     { 48.0, 38.0, 58.0, 40.0, 54.6, 44.0 },   /* Sven Smart RM2000 */
+                                     { 72.0,  1.5,  3.0, 1.74, 2.37, 1.82 },   /* Effekta RM2000MH */
+                                     { 96.0,  1.5,  3.0, 1.63, 2.29,  1.8 },   /* Ablerex MS3000RT (LB at 25% charge) */
+                                     {  0.0,  0.0,  0.0,  0.0,  0.0,  0.0 }};  /* END OF DATA */
 
 /* Defined in upsdrv_initinfo */
 static float battvolt_empty = -1;  /* unknown */
@@ -129,7 +141,7 @@ static float ivolt_min = INT_MAX;  /* unknown */
 static float ivolt_max = -1;       /* unknown */
 
 /* In minutes: */
-static short start_delay = 2;     /* wait this amount of time to come back online */
+static short start_delay = 2;     /* wait at least this amount of time before coming back online */
 static short shutdown_delay = 0;  /* wait until going offline */
 
 /* In percentage: */
@@ -142,10 +154,10 @@ static char watchdog_timeout = 1;  /* in minutes */
 static char *copy_field(char* dest, char *src, int field_len);
 static float get_battery_charge(float battvolt);
 static int set_battery_params(float volt_nominal, float volt_now);
-static int check_ups(void);
-static int get_ups_info(UPSInfo *info);
-static int get_firmware_values(FirmwareValues *values);
-static int run_query(QueryValues *values);
+static int check_ups(QueryValues_t *status);
+static int get_ups_info(UPSInfo_t *info);
+static int get_firmware_values(FirmwareValues_t *values);
+static int run_query(QueryValues_t *values);
 
 int instcmd(const char *cmdname, const char *extra);
 int setvar(const char *varname, const char *val);
@@ -241,49 +253,63 @@ static int set_battery_params(float volt_nominal, float volt_now)
 }
 
 
-static int check_ups(void)
+/*
+ * The "status" parameter is left unchanged on failure.
+ */
+static int check_ups(QueryValues_t *status)
 {
-	char buffer[RECV_BUFFER_LEN];
-	int ret;
-
-	upsdebugx(2, "Sending \"Q1\" command...");
-	ser_send_pace(upsfd, SEND_PACE, "Q1%c", ENDCHAR);
-	ret = ser_get_line(upsfd, buffer, RECV_BUFFER_LEN, ENDCHAR, IGNCHARS, READ_TIMEOUT, 0);
-	if (ret < Q1_CMD_REPLY_LEN) {
-		upsdebugx(2, "Wrong answer to \"Q1\" command.");
+	QueryValues_t values;
 
+	if (run_query(&values) < 0) {
 		return -1;
 	}
-	upsdebugx(2, "\"Q1\" command successful.");
+
+	memcpy(status, &values, sizeof(values));
 
 	return 0;
 }
 
 
-static int get_ups_info(UPSInfo *info)
+static int get_ups_info(UPSInfo_t *info)
 {
 	char buffer[RECV_BUFFER_LEN];
 	char *anchor;
 	int ret;
 
-	upsdebugx(1, "Asking for UPS information (\"I\" command)...");
+	upsdebugx(2, "Asking for UPS information [I]...");
 	ser_send_pace(upsfd, SEND_PACE, "I%c", ENDCHAR);
 	ret = ser_get_line(upsfd, buffer, RECV_BUFFER_LEN, ENDCHAR, IGNCHARS, READ_TIMEOUT, 0);
-	if (ret < I_CMD_REPLY_LEN) {
-		upsdebugx(1, "UPS doesn't return any information about itself.");
+
+	if (ret < 0) {
+		upsdebugx(2, "I => FAILED [timeout]");
 
 		return -1;
 	}
 
-	upsdebugx(3, "UPS information: %s", buffer);
+	if (ret < I_CMD_REPLY_LEN) {
+		upsdebugx(2, "I => FAILED [short read]");
+		upsdebug_hex(3, "I detail", (unsigned char *)buffer, ret);
 
-	memset(info, 0, sizeof(UPSInfo));
+		return -1;
+	}
+
+	if (buffer[0] != '#') {
+		upsdebugx(2, "I => FAILED [invalid start character]");
+		upsdebug_hex(3, "I detail", (unsigned char *)buffer, ret);
+
+		return -1;
+	}
+
+	upsdebugx(2, "I => OK [%s]", buffer);
+
+	memset(info, 0, sizeof(UPSInfo_t));
 
 	/*
 	 * Get the manufacturer, model and version fields, skipping
-	 * the separator character that sits between them.
+	 * the separator character that sits between them, as well as
+	 * the first character (the control character, always a '#').
 	 */
-	anchor = copy_field(info->mfr, buffer, UPS_MFR_CHARS);
+	anchor = copy_field(info->mfr, &buffer[1], UPS_MFR_CHARS);
 	anchor = copy_field(info->model, anchor + 1, UPS_MODEL_CHARS);
 	copy_field(info->version, anchor + 1, UPS_VERSION_CHARS);
 
@@ -291,47 +317,86 @@ static int get_ups_info(UPSInfo *info)
 }
 
 
-static int get_firmware_values(FirmwareValues *values)
+static int get_firmware_values(FirmwareValues_t *values)
 {
 	char buffer[RECV_BUFFER_LEN];
 	int ret;
 
-	upsdebugx(1, "Asking for UPS power ratings (\"F\" command)...");
+	upsdebugx(2, "Asking for UPS power ratings [F]...");
 	ser_send_pace(upsfd, SEND_PACE, "F%c", ENDCHAR);
 	ret = ser_get_line(upsfd, buffer, RECV_BUFFER_LEN, ENDCHAR, IGNCHARS, READ_TIMEOUT, 0);
-	if (ret < F_CMD_REPLY_LEN) {
-		upsdebugx(1, "UPS doesn't return any information about its power ratings.");
+
+	if (ret < 0) {
+		upsdebugx(2, "F => FAILED [timeout]");
 
 		return -1;
 	}
 
-	upsdebugx(3, "UPS power ratings: %s", buffer);
+	if (ret < F_CMD_REPLY_LEN) {
+		upsdebugx(2, "F => FAILED [short read]");
+		upsdebug_hex(3, "F detail", (unsigned char *)buffer, ret);
 
-	sscanf(buffer, "%f %f %f %f", &values->volt, &values->current,
+		return -1;
+	}
+
+
+	if (buffer[0] != '#') {
+		upsdebugx(2, "F => FAILED [invalid start character]");
+		upsdebug_hex(3, "F detail", (unsigned char *)buffer, ret);
+
+		return -1;
+	}
+
+	upsdebugx(2, "F => OK [%s]", buffer);
+
+	sscanf(buffer, "#%f %f %f %f", &values->volt, &values->current,
 	       &values->battvolt, &values->freq);
 
 	return 0;
 }
 
 
-static int run_query(QueryValues *values)
+static int run_query(QueryValues_t *values)
 {
 	char buffer[RECV_BUFFER_LEN];
+	char temperature[8];
 	int ret;
 
-	upsdebugx(1, "Asking for UPS status (\"Q1\" command)...");
+	upsdebugx(2, "Asking for UPS status [Q1]...");
 	ser_send_pace(upsfd, SEND_PACE, "Q1%c", ENDCHAR);
 	ret = ser_get_line(upsfd, buffer, RECV_BUFFER_LEN, ENDCHAR, IGNCHARS, READ_TIMEOUT, 0);
-	if (ret < Q1_CMD_REPLY_LEN) {
-		upsdebugx(1, "UPS doesn't return any information about its status.");
+
+	if (ret < 0) {
+		upsdebugx(2, "Q1 => FAILED [timeout]");
 
 		return -1;
 	}
 
-	upsdebugx(3, "UPS status: %s", buffer);
+	if (ret < Q1_CMD_REPLY_LEN) {
+		upsdebugx(2, "Q1 => FAILED [short read]");
+		upsdebug_hex(3, "Q1 detail", (unsigned char *)buffer, ret);
 
-	sscanf(buffer, "%f %f %f %f %f %f %f %s", &values->ivolt, &values->fvolt, &values->ovolt,
-	       &values->load, &values->freq, &values->battvolt, &values->temp, values->flags);
+		return -1;
+	}
+
+	if (buffer[0] != '(') {
+		upsdebugx(2, "Q1 => FAILED [invalid start character]");
+		upsdebug_hex(3, "Q1 detail", (unsigned char *)buffer, ret);
+
+		return -1;
+	}
+
+	upsdebugx(2, "Q1 => OK [%s]", buffer);
+
+	sscanf(buffer, "(%f %f %f %f %f %f %s %s", &values->ivolt, &values->fvolt, &values->ovolt,
+	       &values->load, &values->freq, &values->battvolt, temperature, values->flags);
+
+	/*
+	 * UPS temperature must be parsed on its own. Some models put
+	 * something other than a float in this field, meaning there is no
+	 * temperature reading available.
+	 */
+	values->temp = atof(temperature);
 
 	return 0;
 }
@@ -341,14 +406,18 @@ void upsdrv_initinfo(void)
 {
 	int i;
 	int success = 0;
-	FirmwareValues values;
-	QueryValues query;
-	UPSInfo info;
+	FirmwareValues_t values;
+	QueryValues_t status;
+	UPSInfo_t info;
 
+	dstate_setinfo("driver.version.internal", "%s", DRV_VERSION);
+
+	/*
+	 * UPS detection sequence.
+	 */
 	upsdebugx(1, "Starting UPS detection process...");
 	for (i = 0; i < IDENT_MAXTRIES; i++) {
-		upsdebugx(2, "Attempting to detect the UPS...");
-		if (check_ups() == 0) {
+		if (check_ups(&status) == 0) {
 			success++;
 		}
 	}
@@ -358,20 +427,18 @@ void upsdrv_initinfo(void)
 
 	if (success < IDENT_MINSUCCESS) {
 		if (success > 0) {
-			fatalx("The UPS is supported, but the connection is too unreliable. Try checking the cable for defects.");
+			fatalx(EXIT_FAILURE, "The UPS is supported, but the connection is too unreliable. Try checking the cable for defects.");
 		} else {
-			fatalx("Megatec protocol UPS not detected.");
+			fatalx(EXIT_FAILURE, "Megatec protocol UPS not detected.");
 		}
 	}
 
-	dstate_setinfo("driver.version.internal", "%s", DRV_VERSION);
-
 	/*
 	 * Try to identify the UPS.
 	 */
 	if (get_ups_info(&info) >= 0) {
 		char model[UPS_MODEL_CHARS + UPS_VERSION_CHARS + 2];
-		sprintf(model, "%s %s", info.model, info.version);
+		snprintf(model, sizeof(model), "%s %s", info.model, info.version);
 
 		dstate_setinfo("ups.mfr", "%s", getval("mfr") ? getval("mfr") : info.mfr);
 		dstate_setinfo("ups.model", "%s", getval("model") ? getval("model") : model);
@@ -386,27 +453,43 @@ void upsdrv_initinfo(void)
 
 	dstate_setinfo("ups.serial", "%s", getval("serial") ? getval("serial") : "unknown");
 
+	/*
+	 * Set battery-related values.
+	 */
 	if (get_firmware_values(&values) >= 0) {
 		dstate_setinfo("output.voltage.nominal", "%.1f", values.volt);
 		dstate_setinfo("battery.voltage.nominal", "%.1f", values.battvolt);
 
-		if (run_query(&query) < 0) {
-			fatalx("Error reading status from UPS!");
-		}
-
-		if (set_battery_params(values.battvolt, query.battvolt) < 0) {
+		if (set_battery_params(values.battvolt, status.battvolt) < 0) {
 			upslogx(LOG_NOTICE, "This UPS has an unsupported combination of battery voltage/number of batteries.");
 		}
 	}
 
+	if (getval("battvolts")) {
+		upsdebugx(2, "Parameter [battvolts]: [%s]", getval("battvolts"));
+
+		if (sscanf(getval("battvolts"), "%f:%f", &battvolt_empty, &battvolt_full) != 2) {
+			fatalx(EXIT_FAILURE, "Error in \"battvolts\" parameter.");
+		}
+
+	    upslogx(LOG_NOTICE, "Overriding battery voltage interval [%.1fV, %.1fV].", battvolt_empty, battvolt_full);
+	}
+
+	if (battvolt_empty < 0 || battvolt_full < 0) {
+		upslogx(LOG_NOTICE, "Cannot calculate charge percentage for this UPS.");
+	}
+
 	if (getval("lowbatt")) {
 		if (battvolt_empty < 0 || battvolt_full < 0) {
-			upslogx(LOG_NOTICE, "Cannot calculate charge percentage for this UPS. Ignoring \"lowbatt\" parameter.");
+			upslogx(LOG_NOTICE, "Ignoring \"lowbatt\" parameter.");
 		} else {
 			lowbatt = CLAMP(atof(getval("lowbatt")), 0, 100);
 		}
 	}
 
+	/*
+	 * Set the restart and shutdown delays.
+	 */
 	if (getval("ondelay")) {
 		start_delay = CLAMP(atoi(getval("ondelay")), 0, MAX_START_DELAY);
 	}
@@ -415,16 +498,8 @@ void upsdrv_initinfo(void)
 		shutdown_delay = CLAMP(atoi(getval("offdelay")), 0, MAX_SHUTDOWN_DELAY);
 	}
 
-	/*
-	 * Register the available variables.
-	 */
 	dstate_setinfo("ups.delay.start", "%d", start_delay);
-	dstate_setflags("ups.delay.start", ST_FLAG_RW | ST_FLAG_STRING);
-	dstate_setaux("ups.delay.start", MAX_START_DELAY_LEN);
-
 	dstate_setinfo("ups.delay.shutdown", "%d", shutdown_delay);
-	dstate_setflags("ups.delay.shutdown", ST_FLAG_RW | ST_FLAG_STRING);
-	dstate_setaux("ups.delay.shutdown", MAX_SHUTDOWN_DELAY_LEN);
 
 	/*
 	 * Register the available instant commands.
@@ -453,13 +528,13 @@ void upsdrv_initinfo(void)
 
 void upsdrv_updateinfo(void)
 {
-	QueryValues query;
+	QueryValues_t query;
 	float charge;
 
 	if (run_query(&query) < 0) {
 		/*
 		 * Query wasn't successful (we got some weird
-		 * response), however we won't fatalx() as this
+		 * response), however we won't fatalx(EXIT_FAILURE, ) as this
 		 * happens sometimes when the ups is offline.
 		 *
 		 * Some fault tolerance is good, we just assume
@@ -481,6 +556,8 @@ void upsdrv_updateinfo(void)
 	charge = get_battery_charge(query.battvolt);
 	if (charge >= 0) {
 		dstate_setinfo("battery.charge", "%.1f", charge);
+
+		upsdebugx(2, "Calculated battery charge: %.1f%%", charge);
 	}
 
 	status_init();
@@ -540,10 +617,13 @@ void upsdrv_updateinfo(void)
 
 void upsdrv_shutdown(void)
 {
-	upslogx(LOG_INFO, "Shutting down UPS immediately.");
+	int s_wait = getval("offdelay") ? CLAMP(atoi(getval("offdelay")), 0, MAX_SHUTDOWN_DELAY) : shutdown_delay;
+	int r_wait = getval("ondelay") ? CLAMP(atoi(getval("ondelay")), 0, MAX_START_DELAY) : start_delay;
+
+	upslogx(LOG_INFO, "Shutting down UPS.");
 
 	ser_send_pace(upsfd, SEND_PACE, "C%c", ENDCHAR);
-	ser_send_pace(upsfd, SEND_PACE, "S%02dR%04d%c", shutdown_delay, start_delay, ENDCHAR);
+	ser_send_pace(upsfd, SEND_PACE, "S%02dR%04d%c", s_wait, r_wait, ENDCHAR);
 }
 
 
@@ -687,32 +767,6 @@ int instcmd(const char *cmdname, const char *extra)
 
 int setvar(const char *varname, const char *val)
 {
-	int delay;
-
-	if (sscanf(val, "%d", &delay) != 1) {
-		return STAT_SET_UNKNOWN;
-	}
-
-	if (strcasecmp(varname, "ups.delay.start") == 0) {
-		delay = CLAMP(delay, 0, MAX_START_DELAY);
-		start_delay = delay;
-		dstate_setinfo("ups.delay.start", "%d", delay);
-
-		dstate_dataok();
-
-		return STAT_SET_HANDLED;
-	}
-
-	if (strcasecmp(varname, "ups.delay.shutdown") == 0) {
-		delay = CLAMP(delay, 0, MAX_SHUTDOWN_DELAY);
-		shutdown_delay = delay;
-		dstate_setinfo("ups.delay.shutdown", "%d", delay);
-
-		dstate_dataok();
-
-		return STAT_SET_HANDLED;
-	}
-
 	return STAT_SET_UNKNOWN;
 }
 
@@ -728,15 +782,20 @@ void upsdrv_makevartable(void)
 	addvar(VAR_VALUE, "model", "Model name");
 	addvar(VAR_VALUE, "serial", "UPS serial number");
 	addvar(VAR_VALUE, "lowbatt", "Low battery level (%)");
-	addvar(VAR_VALUE, "ondelay", "Delay before UPS startup (minutes)");
+	addvar(VAR_VALUE, "ondelay", "Minimum delay before UPS startup (minutes)");
 	addvar(VAR_VALUE, "offdelay", "Delay before UPS shutdown (minutes)");
+	addvar(VAR_VALUE, "battvolts", "Battery voltages (empty:full)");
+
+	megatec_subdrv_makevartable();
 }
 
 
 void upsdrv_banner(void)
 {
-	printf("Network UPS Tools - Megatec protocol driver %s (%s)\n", DRV_VERSION, UPS_VERSION);
-	printf("Carlos Rodrigues (c) 2003-2006\n\n");
+	printf("Network UPS Tools %s - Megatec protocol driver %s [%s]\n", UPS_VERSION, DRV_VERSION, progname);
+	printf("Carlos Rodrigues (c) 2003-2007\n\n");
+
+	megatec_subdrv_banner();
 }
 
 
@@ -744,11 +803,16 @@ void upsdrv_initups(void)
 {
 	upsfd = ser_open(device_path);
 	ser_set_speed(upsfd, device_path, B2400);
+
+	/* Some UPS models need this. */
+	ser_set_dtr(upsfd, 1);
+	ser_set_rts(upsfd, 0);
 }
 
 
 void upsdrv_cleanup(void)
 {
+	ser_set_dtr(upsfd, 0);
 	ser_close(upsfd, device_path);
 }
 
