@@ -41,7 +41,7 @@ static void nutusb_open_error(const char *port)
 	printf(" - Run this driver as another user (upsdrvctl -u or 'user=...' in ups.conf).\n");
 	printf("   See upsdrvctl(8) and ups.conf(5).\n\n");
 
-	fatalx("Fatal error: unusable configuration");
+	fatalx(EXIT_FAILURE, "Fatal error: unusable configuration");
 }
 
 static usb_dev_handle *open_powerware_usb()
@@ -55,7 +55,7 @@ static usb_dev_handle *open_powerware_usb()
     
 		for (dev = bus->devices; dev; dev = dev->next)
 		{
-			/* XXX Check for POWERWARE 3105 or 3110 ... other models??? */
+			/* XXX Check for POWERWARE 3105, 3110, 5115 (500i USB) ...or  other models??? */
 			if (dev->descriptor.bDeviceClass == USB_CLASS_PER_INTERFACE &&
 			    (dev->descriptor.idVendor == 0x0592 ||
 			     dev->descriptor.idVendor == 0x06da) &&
