@@ -7,7 +7,7 @@
  *    This model is based on PowerCom (www.powercom.com) models.
  *  -Socomec Sicon Egys 420
  *
- * $Id: powercom.c 440 2006-07-10 12:33:14Z aquette $
+ * $Id: powercom.c 1127 2007-09-28 17:17:47Z adkorte-guest $
  *
  * Copyrights:
  * (C) 2002 Simon Rozman <simon@rozman.net>
@@ -153,14 +153,11 @@ static int instcmd (const char *cmdname, const char *extra)
  */
 static void dtr0rts1 (void)
 {
-	int dtr_bit = TIOCM_DTR;
-	int rts_bit = TIOCM_RTS;
-
 	upsdebugx(2, "DTR => 0, RTS => 1");
 
 	/* set DTR to low and RTS to high */
-	ioctl(upsfd, TIOCMBIC, &dtr_bit);
-	ioctl(upsfd, TIOCMBIS, &rts_bit);
+	ser_set_dtr(upsfd, 0);
+	ser_set_rts(upsfd, 1);
 }
 
 /* set DTR line on a serial port for KIN525AP
@@ -168,12 +165,10 @@ static void dtr0rts1 (void)
  */
 static void dtr1 (void)
 {
-	int dtr_bit = TIOCM_DTR;
-
 	upsdebugx(2, "DTR => 1");
 
 	/* set DTR to high */
-	ioctl(upsfd, TIOCMBIS, &dtr_bit);
+	ser_set_dtr(upsfd, 1);
 }
 
 /* clear any flow control */
@@ -296,7 +291,7 @@ void upsdrv_updateinfo(void)
 		        types[type].battpct[1] * tmp + types[type].battpct[2] :
 			types[type].battpct[3] * raw_data[BATTERY_CHARGE] +
 			                                   types[type].battpct[4]);
-	upsdebugx(2, "battery.charge: %s", dstate_getinfo("battery_charge"));
+	upsdebugx(2, "battery.charge: %s", dstate_getinfo("battery.charge"));
 
 	/* input.voltage */	
 	upsdebugx(3, "input.voltage (raw data): [raw: %u]",
@@ -310,7 +305,7 @@ void upsdrv_updateinfo(void)
 	upsdebugx(2, "input.voltage: %s", dstate_getinfo("input.voltage"));
 	
 	/* output.voltage */	
-	upsdebugx(3, "input.voltage (raw data): [raw: %u]",
+	upsdebugx(3, "output.voltage (raw data): [raw: %u]",
 	                            raw_data[OUTPUT_VOLTAGE]);
 	dstate_setinfo("output.voltage", "%03.1f",
 	    linevoltage >= 220 ?
