@@ -35,6 +35,16 @@
  *   - put all UPS commands in header file (for easy retrieval)
  *   - remove log errors when instant commands can't be handled
  *     (doesn't work reliably on at least one flavor)
+ *  20061229/Revision 1.1 - Arjen de Korte <arjen@de-korte.org>
+ *   - flush the serial input buffer before sending commands
+ *     (to get rid of unsollicited serial PnP replies)
+ *  20070211/Revision 1.2 - Arjen de Korte <arjen@de-korte.org>
+ *   - wait for three consecutive failed polls before declaring
+ *     data stale in order not to bother clients with temporary
+ *     problems
+ *  20070304/Revision 1.3 - Arjen de Korte <arjen@de-korte.org>
+ *   - in battery test mode (CAL state) stop the test when the
+ *     the low battery state is reached
  *
  * Copyright (C) 2003-2006  Arjen de Korte <arjen@de-korte.org>
  *
@@ -55,24 +65,31 @@
 
 #include <stdlib.h>
 #include <ctype.h>
-#include <sys/ioctl.h>
 
 #include "main.h"
 #include "serial.h"
 #include "safenet.h"
 
-#define DRV_VERSION	"1.0"
+#define DRV_VERSION	"1.4"
 
 /*
  * Here we keep the last known status of the UPS
  */
-static safenet_u	ups;
+static union	{
+	char			reply[10];
+	struct safenet		status;
+} ups;
 
 static int safenet_command(const char *command)
 {
 	char	reply[32];
 	int	i;
 
+	/*
+	 * Get rid of whatever is in the in- and output buffers.
+	 */
+	ser_flush_io(upsfd);
+
 	/*
 	 * Send the command and read back the status line. When we just send
 	 * a status polling command, it will return the actual status.
@@ -91,39 +108,36 @@ static int safenet_command(const char *command)
 	 * We check if the reply looks like a valid status.
 	 */
 
-	if ((strlen(reply) != 11) || (reply[0] != '$') || (strspn(reply+1, "AB") != 10))
-	{
+	if ((strlen(reply) != 11) || (reply[0] != '$') || (strspn(reply+1, "AB") != 10)) {
 		return(-1);
 	}
 
-	for (i=0; i<10; i++)
-	{
+	for (i=0; i<10; i++) {
 		ups.reply[i] = ((reply[i+1] == 'B') ? 1 : 0);
 	}
 
+	return(0);
+}
+
+static void safenet_update()
+{
 	status_init();
 
-	if (ups.status.onbattery)
-	{
+	if (ups.status.onbattery) {
 		status_set("OB");
-	}
-	else
-	{
+	} else {
 		status_set("OL");
 	}
 
-	if (ups.status.batterylow)
-	{
+	if (ups.status.batterylow) {
 		status_set("LB");
 	}
 
-	if (ups.status.overload)
-	{
+	if (ups.status.overload) {
 		status_set("OVER");
 	}
 
-	if (ups.status.batteryfail)
-	{
+	if (ups.status.batteryfail) {
 		status_set("RB");
 	}
 
@@ -134,8 +148,7 @@ static int safenet_command(const char *command)
 	 * backup in case of mains failure) we give it this status. Introduce a new
 	 * status here?
 	 */
-	if (ups.status.systemfail)
-	{
+	if (ups.status.systemfail) {
 		status_set("BYPASS");
 	}
 
@@ -145,23 +158,33 @@ static int safenet_command(const char *command)
 	 * test may indicate a battery failure, so this is the closest we can get.
 	 * Introduce a new status here?
 	 */
-	if (ups.status.systemtest)
-	{
+	if (ups.status.systemtest) {
 		status_set("CAL");
 	}
 
 	status_commit();
-
-	return(0);
 }
 
 static int instcmd(const char *cmdname, const char *extra)
 {
+	if (!strcasecmp(cmdname, "beeper.off")) {
+		/* compatibility mode for old command */
+		upslogx(LOG_WARNING,
+			"The 'beeper.off' command has been renamed to 'beeper.mute' for this driver");
+		return instcmd("beeper.mute", NULL);
+	}
+
+	if (!strcasecmp(cmdname, "beeper.on")) {
+		/* compatibility mode for old command */
+		upslogx(LOG_WARNING,
+			"The 'beeper.on' command has been renamed to 'beeper.enable'");
+		return instcmd("beeper.enable", NULL);
+	}
+
 	/*
 	 * Start the UPS selftest
 	 */
-	if (!strcasecmp(cmdname, "test.battery.start"))
-	{
+	if (!strcasecmp(cmdname, "test.battery.start")) {
 		safenet_command(COM_BATT_TEST);
 		return STAT_INSTCMD_HANDLED;
 	}
@@ -169,17 +192,15 @@ static int instcmd(const char *cmdname, const char *extra)
 	/*
 	 * Stop the UPS selftest
 	 */
-	if (!strcasecmp(cmdname, "test.battery.stop"))
-	{
-		safenet_command(COM_ABORT_TEST);
+	if (!strcasecmp(cmdname, "test.battery.stop")) {
+		safenet_command(COM_STOP_TEST);
 		return STAT_INSTCMD_HANDLED;
 	}
 
 	/*
 	 * Start simulated mains failure
 	 */
-	if (!strcasecmp (cmdname, "test.failure.start"))
-	{
+	if (!strcasecmp (cmdname, "test.failure.start")) {
 		safenet_command(COM_MAINS_TEST);
 		return STAT_INSTCMD_HANDLED;
 	}
@@ -187,35 +208,48 @@ static int instcmd(const char *cmdname, const char *extra)
 	/*
 	 * Stop simulated mains failure
 	 */
-	if (!strcasecmp (cmdname, "test.failure.stop"))
-	{
-		safenet_command(COM_ABORT_TEST);
+	if (!strcasecmp (cmdname, "test.failure.stop")) {
+		safenet_command(COM_STOP_TEST);
 		return STAT_INSTCMD_HANDLED;
 	}
 
 	/*
 	 * If beeper is off, toggle beeper state (so it should be ON after this)
 	 */
-	if (!strcasecmp(cmdname, "beeper.on"))
-	{
-		if (ups.status.silenced) safenet_command(COM_TOGGLE_BEEP);
+	if (!strcasecmp(cmdname, "beeper.enable")) {
+		if (ups.status.silenced) {
+			safenet_command(COM_TOGGLE_BEEP);
+		}
+
 		return STAT_INSTCMD_HANDLED;
 	}
 
 	/*
 	 * If beeper is not off, toggle beeper state (so it should be OFF after this)
+	 * Unfortunately, this only mutes the beeper, it turns back on for the next
+	 * event automatically (no way to stop this, besides side cutters)
 	 */
-	if (!strcasecmp(cmdname, "beeper.off"))
-	{
-		if (!ups.status.silenced) safenet_command(COM_TOGGLE_BEEP);
+	if (!strcasecmp(cmdname, "beeper.mute")) {
+		if (!ups.status.silenced) {
+			safenet_command(COM_TOGGLE_BEEP);
+		}
+
+		return STAT_INSTCMD_HANDLED;
+	}
+
+	/*
+	 * Toggle beeper state unconditionally
+	 */
+	if (!strcasecmp(cmdname, "beeper.toggle")) {
+		safenet_command(COM_TOGGLE_BEEP);
+
 		return STAT_INSTCMD_HANDLED;
 	}
 
 	/*
 	 * Shutdown immediately and wait for the power to return
 	 */
-	if (!strcasecmp(cmdname, "shutdown.return"))
-	{
+	if (!strcasecmp(cmdname, "shutdown.return")) {
 		safenet_command(SHUTDOWN_RETURN);
 		return STAT_INSTCMD_HANDLED;
 	}
@@ -223,8 +257,7 @@ static int instcmd(const char *cmdname, const char *extra)
 	/*
 	 * Shutdown immediately and reboot after 1 minute
 	 */
-	if (!strcasecmp(cmdname, "shutdown.reboot"))
-	{
+	if (!strcasecmp(cmdname, "shutdown.reboot")) {
 		safenet_command(SHUTDOWN_REBOOT);
 		return STAT_INSTCMD_HANDLED;
 	}
@@ -232,8 +265,7 @@ static int instcmd(const char *cmdname, const char *extra)
 	/*
 	 * Shutdown in 20 seconds and reboot after 1 minute
 	 */
-	if (!strcasecmp(cmdname, "shutdown.reboot.graceful"))
-	{
+	if (!strcasecmp(cmdname, "shutdown.reboot.graceful")) {
 		safenet_command(GRACEFUL_REBOOT);
 		return STAT_INSTCMD_HANDLED;
 	}
@@ -244,7 +276,7 @@ static int instcmd(const char *cmdname, const char *extra)
 
 void upsdrv_initinfo(void)
 {
-	int	i, retry = 3;
+	int	retry = 3;
 	char	*v;
 
 	dstate_setinfo("driver.version.internal", "%s", DRV_VERSION);
@@ -255,27 +287,21 @@ void upsdrv_initinfo(void)
 	 * Very crude hardware detection. If an UPS is attached, it will set DSR
 	 * to 1. Bail out if it isn't.
 	 */
-	ioctl(upsfd, TIOCMGET, &i);
-	if ((i & TIOCM_DSR) == 0)
-	{
-		fatalx("Serial cable problem or nothing attached to %s", device_path);
+	if (!ser_get_dsr(upsfd)) {
+		fatalx(EXIT_FAILURE, "Serial cable problem or nothing attached to %s", device_path);
 	}
 
-	/*
-	 * Get rid of PnP garbage that may be still left in the buffer.
-	 */
-	ser_flush_in(upsfd, "", 0);
-
 	/*
 	 * Initialize the serial interface of the UPS by sending the magic
 	 * string. If it does not respond with a valid status reply,
 	 * display an error message and give up.
 	 */
-	while (safenet_command(COM_INITIALIZE))
-	{
-		if (--retry) continue;
+	while (safenet_command(COM_INITIALIZE)) {
+		if (--retry) {
+			continue;
+		}
 
-		fatalx("SafeNet protocol compatible UPS not found on %s", device_path);
+		fatalx(EXIT_FAILURE, "SafeNet protocol compatible UPS not found on %s", device_path);
 	}
 
 	/*
@@ -295,6 +321,9 @@ void upsdrv_initinfo(void)
 	dstate_addcmd ("test.failure.stop");
 	dstate_addcmd ("beeper.on");
 	dstate_addcmd ("beeper.off");
+	dstate_addcmd ("beeper.enable");
+	dstate_addcmd ("beeper.mute");
+	dstate_addcmd ("beeper.toggle");
 	dstate_addcmd ("shutdown.return");
 	dstate_addcmd ("shutdown.reboot");
 	dstate_addcmd ("shutdown.reboot.graceful");
@@ -317,12 +346,15 @@ void upsdrv_updateinfo(void)
 {
 	char	command[] = COM_POLL_STAT;
 	int	i;
+	static int	retry = 0;
 
 	/*
 	 * Fill the command portion with random characters from the range
 	 * [A...J].
 	 */
-	for (i = 1; i < 12; i++) command[i] = (random() % 10) + 'A';
+	for (i = 1; i < 12; i++) {
+		command[i] = (random() % 10) + 'A';
+	}
 
 	/*
 	 * Find which character must be an 'L' and put it there.
@@ -332,29 +364,34 @@ void upsdrv_updateinfo(void)
 	/*
 	 * Do a status poll.
 	 */
-	if (safenet_command(command))
-	{
-		upsdebugx(1, "UPS serial FAIL");
+	if (safenet_command(command)) {
 		ser_comm_fail("Status read failed");
-		dstate_datastale();
-	}	
-	else
-	{
-		upsdebugx(1, "UPS serial OK");
-		ser_comm_good();
-		dstate_dataok();
+
+		if (retry < 2) {
+			retry++;
+		} else {
+			dstate_datastale();
+		}
+
+		return;
 	}
+
+	ser_comm_good();
+	retry = 0;
+
+	if (ups.status.systemtest && ups.status.batterylow) {
+		safenet_command(COM_STOP_TEST);
+	}
+
+	safenet_update();
+
+	dstate_dataok();
 }
 
 void upsdrv_shutdown(void)
 {
 	int	retry = 3;
 
-	/*
-	 * Get rid of PnP garbage that may be still left in the buffer.
-	 */
-	ser_flush_in(upsfd, "", 0);
-
 	/*
 	 * Since we may have arrived here before the hardware is initialized,
 	 * try to initialize it here.
@@ -363,11 +400,12 @@ void upsdrv_shutdown(void)
 	 * string. If it does not respond with a valid status reply,
 	 * display an error message and give up.
 	 */
-	while (safenet_command(COM_INITIALIZE))
-	{
-		if (--retry) continue;
+	while (safenet_command(COM_INITIALIZE)) {
+		if (--retry) {
+			continue;
+		}
 
-		fatalx("SafeNet protocol compatible UPS not found on %s", device_path);
+		fatalx(EXIT_FAILURE, "SafeNet protocol compatible UPS not found on %s", device_path);
 	}
 
 	/*
@@ -375,21 +413,9 @@ void upsdrv_shutdown(void)
 	 * different shutdown command depending on the line status, so
 	 * we need to check the status of the UPS here.
 	 */
-	if (ups.status.onbattery)
-	{
-		/*
-		 * Kill the inverter and wait for the things to
-		 * come.
-		 */
+	if (ups.status.onbattery) {
 		safenet_command(SHUTDOWN_RETURN);
-	}
-	else
-	{
-		/*
-		 * Apparently we're not running on battery right
-		 * now. Let's cycle the UPS and try to prevent a
-		 * lockup.
-		 */
+	} else {
 		safenet_command(SHUTDOWN_REBOOT);
 	}
 }
@@ -408,14 +434,10 @@ void upsdrv_makevartable(void)
 void upsdrv_banner(void)
 {
 	printf("Network UPS Tools - Generic SafeNet UPS driver %s (%s)\n\n", DRV_VERSION, UPS_VERSION);
-	experimental_driver = 0;
 }
 
 void upsdrv_initups(void)
 {
-	int dtr_bit = TIOCM_DTR;
-	int rts_bit = TIOCM_RTS;
-
 	/*
 	 * Open and lock the serial port and set the speed to 1200 baud.
 	 */
@@ -425,11 +447,12 @@ void upsdrv_initups(void)
 	/*
 	 * Set DTR and clear RTS to provide power for the serial interface.
 	 */
-	ioctl(upsfd, TIOCMBIS, &dtr_bit);
-	ioctl(upsfd, TIOCMBIC, &rts_bit);
+	ser_set_dtr(upsfd, 1);
+	ser_set_rts(upsfd, 0);
 }
 
 void upsdrv_cleanup(void)
 {
+	ser_set_dtr(upsfd, 0);
 	ser_close(upsfd, device_path);
 }
