@@ -27,6 +27,7 @@
 #include <ctype.h>
 #include <sys/file.h>
 #include <sys/types.h>
+#include <sys/ioctl.h>
 #include <unistd.h>
 
 #ifdef HAVE_UU_LOCK
@@ -53,7 +54,7 @@ static void ser_open_error(const char *port)
 		printf("Things to try:\n\n");
 		printf(" - Check 'port=' in ups.conf\n\n");
 		printf(" - Check owner/permissions of all parts of path\n\n");
-		fatalx("Fatal error: unusable configuration");
+		fatalx(EXIT_FAILURE, "Fatal error: unusable configuration");
 	}
 
 	user = getpwuid(getuid());
@@ -82,7 +83,7 @@ static void ser_open_error(const char *port)
 	printf(" - Run this driver as another user (upsdrvctl -u or 'user=...' in ups.conf).\n");
 	printf("   See upsdrvctl(8) and ups.conf(5).\n\n");
 
-	fatalx("Fatal error: unusable configuration");
+	fatalx(EXIT_FAILURE, "Fatal error: unusable configuration");
 }
 
 static void lock_set(int fd, const char *port)
@@ -90,7 +91,7 @@ static void lock_set(int fd, const char *port)
 	int	ret;
 
 	if (fd < 0)
-		fatal_with_errno("lock_set: programming error: fd = %d", fd);
+		fatal_with_errno(EXIT_FAILURE, "lock_set: programming error: fd = %d", fd);
 
 	if (do_lock_port == 0)
 		return;
@@ -99,7 +100,7 @@ static void lock_set(int fd, const char *port)
 	ret = uu_lock(xbasename(port));
 
 	if (ret != 0)
-		fatalx("Can't uu_lock %s: %s", xbasename(port), 
+		fatalx(EXIT_FAILURE, "Can't uu_lock %s: %s", xbasename(port), 
 			uu_lockerr(ret));
 
 	return;
@@ -109,7 +110,7 @@ static void lock_set(int fd, const char *port)
 	ret = flock(fd, LOCK_EX | LOCK_NB);
 
 	if (ret != 0)
-		fatalx("%s is locked by another process", port);
+		fatalx(EXIT_FAILURE, "%s is locked by another process", port);
 
 	return;
 
@@ -120,7 +121,7 @@ static void lock_set(int fd, const char *port)
 	ret = lockf(fd, F_TLOCK, 0L);
 
 	if (ret != 0)
-		fatalx("%s is locked by another process", port);
+		fatalx(EXIT_FAILURE, "%s is locked by another process", port);
 
 	return;
 
@@ -148,7 +149,7 @@ int ser_set_speed(int fd, const char *port, speed_t speed)
 	struct	termios	tio;
 
 	if (tcgetattr(fd, &tio) != 0)
-		fatal_with_errno("tcgetattr(%s)", port);
+		fatal_with_errno(EXIT_FAILURE, "tcgetattr(%s)", port);
 
 	tio.c_cflag = CS8 | CLOCAL | CREAD;
 	tio.c_iflag = IGNPAR;
@@ -170,10 +171,58 @@ int ser_set_speed(int fd, const char *port, speed_t speed)
 	return 0;
 }
 
+static int ser_set_control(int fd, int line, int state)
+{
+	if (state) {
+		return ioctl(fd, TIOCMBIS, &line);
+	} else {
+		return ioctl(fd, TIOCMBIC, &line);
+	}
+}
+
+int ser_set_dtr(int fd, int state)
+{
+	return ser_set_control(fd, TIOCM_DTR, state);
+}
+
+int ser_set_rts(int fd, int state)
+{
+	return ser_set_control(fd, TIOCM_RTS, state);
+}
+
+static int ser_get_control(int fd, int line)
+{
+	int	flags;
+
+	ioctl(fd, TIOCMGET, &flags);
+
+	return (flags & line);
+}
+
+int ser_get_dsr(int fd)
+{
+	return ser_get_control(fd, TIOCM_DSR);
+}
+
+int ser_get_cts(int fd)
+{
+	return ser_get_control(fd, TIOCM_CTS);
+}
+
+int ser_get_dcd(int fd)
+{
+	return ser_get_control(fd, TIOCM_CD);
+}
+
+int ser_flush_io(int fd)
+{
+	return tcflush(fd, TCIOFLUSH);
+}
+
 int ser_close(int fd, const char *port)
 {
 	if (fd < 0)
-		fatal_with_errno("ser_close: programming error: fd=%d port=%s", fd, port);
+		fatal_with_errno(EXIT_FAILURE, "ser_close: programming error: fd=%d port=%s", fd, port);
 
 	if (close(fd) != 0)
 		return -1;
