@@ -26,7 +26,6 @@
 
 #include "config.h"
 #include "proto.h"
-#include "version.h"
 #include "common.h"
 #include "upsconf.h"
 
@@ -65,9 +64,7 @@ void do_upsconf_args(char *upsname, char *var, char *val)
 			maxstartdelay = atoi(val);
 
 		if (!strcmp(var, "driverpath")) {
-			if (driverpath)
-				free(driverpath);
-
+			free(driverpath);
 			driverpath = xstrdup(val);
 		}
 
@@ -134,12 +131,12 @@ static void stop_driver(const ups_t *ups)
 	upsdebugx(1, "Stopping UPS: %s", ups->upsname);
 
 	snprintf(pidfn, sizeof(pidfn), "%s/%s-%s.pid", altpidpath(),
-		ups->driver, xbasename(ups->port));
+		ups->driver, ups->upsname);
 	ret = stat(pidfn, &fs);
 
 	if (ret != 0) {
 		snprintf(pidfn, sizeof(pidfn), "%s/%s-%s.pid", altpidpath(),
-			ups->driver, ups->upsname);
+			ups->driver, xbasename(ups->port));
 		ret = stat(pidfn, &fs);
 	}
 
@@ -177,7 +174,7 @@ static void forkexec(const char *prog, char **argv, const ups_t *ups)
 	pid = fork();
 
 	if (pid < 0)
-		fatal_with_errno("fork");
+		fatal_with_errno(EXIT_FAILURE, "fork");
 
 	if (pid != 0) {			/* parent */
 		int	wstat;
@@ -233,7 +230,7 @@ static void forkexec(const char *prog, char **argv, const ups_t *ups)
 	ret = execv(prog, argv);
 
 	/* shouldn't get here */
-	fatal_with_errno("execv");
+	fatal_with_errno(EXIT_FAILURE, "execv");
 }
 
 static void start_driver(const ups_t *ups)
@@ -248,7 +245,7 @@ static void start_driver(const ups_t *ups)
 	ret = stat(dfn, &fs);
 
 	if (ret < 0)
-		fatal_with_errno("Can't start %s", dfn);
+		fatal_with_errno(EXIT_FAILURE, "Can't start %s", dfn);
 
 	upsdebugx(2, "exec: %s -a %s", dfn, ups->upsname);
 
@@ -324,7 +321,7 @@ static void send_one_driver(void (*command)(const ups_t *), const char *upsname)
 	ups_t	*ups = upstable;
 
 	if (!ups)
-		fatalx("Error: no UPS definitions found in ups.conf!\n");
+		fatalx(EXIT_FAILURE, "Error: no UPS definitions found in ups.conf!\n");
 
 	while (ups) {
 		if (!strcmp(ups->upsname, upsname)) {
@@ -335,7 +332,7 @@ static void send_one_driver(void (*command)(const ups_t *), const char *upsname)
 		ups = ups->next;
 	}
 
-	fatalx("UPS %s not found in ups.conf", upsname);
+	fatalx(EXIT_FAILURE, "UPS %s not found in ups.conf", upsname);
 }
 
 /* walk UPS table and send command to all UPSes according to sdorder */
@@ -345,7 +342,7 @@ static void send_all_drivers(void (*command)(const ups_t *))
 	int	i;
 
 	if (!upstable)
-		fatalx("Error: no UPS definitions found in ups.conf");
+		fatalx(EXIT_FAILURE, "Error: no UPS definitions found in ups.conf");
 
 	if (command != &shutdown_driver) {
 		ups = upstable;
@@ -380,21 +377,15 @@ static void exit_cleanup(void)
 	while (tmp) {
 		next = tmp->next;
 
-		if (tmp->driver)
-			free(tmp->driver);
-
-		if (tmp->port)
-			free(tmp->port);
-
-		if (tmp->upsname)
-			free(tmp->upsname);
+		free(tmp->driver);
+		free(tmp->port);
+		free(tmp->upsname);
 		free(tmp);
 
 		tmp = next;
 	}
 
-	if (driverpath)
-		free(driverpath);
+	free(driverpath);
 }
 
 int main(int argc, char **argv)
@@ -407,7 +398,7 @@ int main(int argc, char **argv)
 		UPS_VERSION);
 
 	prog = argv[0];
-	while ((i = getopt(argc, argv, "+htu:r:DV")) != EOF) {
+	while ((i = getopt(argc, argv, "+htu:r:DV")) != -1) {
 		switch(i) {
 			case 'r':
 				pt_root = optarg;
@@ -458,7 +449,7 @@ int main(int argc, char **argv)
 		command = &shutdown_driver;
 
 	if (!command)
-		fatalx("Error: unrecognized command [%s]", argv[0]);
+		fatalx(EXIT_FAILURE, "Error: unrecognized command [%s]", argv[0]);
 
 	driverpath = xstrdup(DRVPATH);	/* set default */
 
