@@ -19,10 +19,18 @@
 
 void user_load(void);
 
+#ifndef	HAVE_IPV6
 int user_checkinstcmd(const struct sockaddr_in *addr,
+#else
+int user_checkinstcmd(const struct sockaddr_storage *addr,
+#endif
 	const char *un, const char *pw, const char *cmd);
 
+#ifndef	HAVE_IPV6
 int user_checkaction(const struct sockaddr_in *addr,
+#else
+int user_checkaction(const struct sockaddr_storage *addr,
+#endif
 	const char *un, const char *pw, const char *action);
 
 void user_flush(void);
